from django.db import models
from django.conf import settings
from multitenancy.subscriptions.models import Subscription
from multitenancy.address.models import Address
# Create your models here.

class Coupon(models.Model):
    code = models.CharField(max_length=15)
    discount = models.DecimalField(max_digits=10, decimal_places=2)
    start_date = models.DateField()
    end_date = models.DateField()
    usage_limit = models.PositiveIntegerField()
    is_active = models.BooleanField(default=True)
    minimum_order_amount = models.DecimalField(max_digits=10, decimal_places=2)
    redeem_by = models.DateField()
    usage_count = models.PositiveIntegerField(default=0)
    # the valid_for field will have to be connected to a Product model
    # valid_for = models.ForeignKey(Subscription, on_delete=models.SET_NULL, null=True, blank=True) 

class Order(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL,null=True, on_delete=models.CASCADE)
    date_created = models.DateField(auto_now_add=True)
    order_number = models.CharField(max_length=100)
    amount = models.DecimalField(max_digits=10, decimal_places=2)
    status = models.CharField(max_length=10, choices=[("completed", "Completed"), ("failed", "Failed")])
    payment_method = models.CharField(max_length=50)
    billing_address = models.ForeignKey(Address, on_delete=models.CASCADE)
    notes = models.TextField(blank=True)
    coupon = models.ForeignKey(Coupon, on_delete=models.SET_NULL, null=True, blank=True)


class OrderItem(models.Model):
    subscription = models.ForeignKey(Subscription, on_delete=models.CASCADE)
    order = models.ForeignKey(Order, on_delete=models.CASCADE)


from account.mixins import LoginRequiredMixin
from django.shortcuts import redirect, render
from django.urls.base import reverse, reverse_lazy
from multitenancy.settings.forms import AddressForm, AdminSettingsForm, GeneralInfoForm, LogoForm
from multitenancy.settings.models import Address, AdminSettings, GeneralInfo, Logo
from multitenancy.admin.views.baseViews import(
    AdminListView,
    AdminDeleteView,
    AdminCreateView,
    AdminUpdateView,
    AdminTemplateView,
    AdminView,
    AdminDetailView
    )

class SettingsIndexView(LoginRequiredMixin ,AdminTemplateView):
    template_name = "multitenancy/settings/settings_list.html"


class SettingsView(LoginRequiredMixin, AdminTemplateView):
    template_name = 'multitenancy/settings/generalsettings_index.html'
    settings_list = []

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        context['logo'] = Logo.load()
        context['address'] = Address.load()
        context['admin_settings'] = AdminSettings.load()
        context['info'] = GeneralInfo.load()
        return context


class UpdateLogoView(LoginRequiredMixin, AdminUpdateView):

    model = Logo
    form_class = LogoForm
    success_url = reverse_lazy('generalsettings_index', urlconf="multitenancy.urls")
    template_name = 'multitenancy/settings/update_logo.html'

    def get_object(self):
        return self.model.objects.first()


class GeneralInfoView(LoginRequiredMixin, AdminUpdateView):

    model = GeneralInfo
    form_class = GeneralInfoForm
    success_url = reverse_lazy('generalsettings_index', urlconf="multitenancy.urls")
    template_name = 'multitenancy/settings/update_info.html'

    def get_object(self):
        return self.model.objects.first()


class AddressView(LoginRequiredMixin, AdminUpdateView):

    model = Address
    form_class = AddressForm
    success_url = reverse_lazy('generalsettings_index', urlconf="multitenancy.urls")
    template_name = 'multitenancy/settings/update_address.html'

    def get_object(self):
        return self.model.objects.first()

class AdminSettingsView(LoginRequiredMixin, AdminUpdateView):

    model = AdminSettings
    form_class = AdminSettingsForm
    success_url = reverse_lazy('generalsettings_index', urlconf="multitenancy.urls")
    template_name = 'multitenancy/settings/update_adminsettings.html'

    def get_object(self):
        return self.model.objects.first()


from django.shortcuts import render
from rest_framework import viewsets
from rest_framework.decorators import api_view
from rest_framework.reverse import reverse
from rest_framework.response import Response
from rest_framework import permissions
from multitenancy.admin.views.baseViews import AdminTemplateView
from multitenancy.billing.forms import InvoiceForm
from multitenancy.billing.serializers import CreditSerializer, InvoiceSerializer, RefundSerializer

from .models import (
    Invoice,
    PaymentGateWay, 
    Payment, 
    Refund,
    Credit
    )

class BillingIndexView(AdminTemplateView):
    template_name = "multitenancy/billing/billing.html"

class InvoiceListView(AdminTemplateView):
    template_name = "multitenancy/billing/invoice_list.html"
    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)

        form = InvoiceForm
        context['form'] = form
        return context


class CreditsListView(AdminTemplateView):
    template_name = "multitenancy/billing/credits_list.html"

class RefundsListView(AdminTemplateView):
    template_name = "multitenancy/billing/refunds_list.html"

class InvoiceViewSet(viewsets.ModelViewSet):
    """
    This viewset automatically provides `list`, `create`, `retrieve`,
    `update` and `destroy` actions.

    Additionally we also provide an extra `highlight` action.
    """
    model = Invoice
    
    serializer_class = InvoiceSerializer
    permission_classes = [permissions.IsAuthenticated,
                          permissions.IsAdminUser]

    def get_queryset(self):
        query = self.request.GET.get("q") 
        query_id = self.request.GET.get("id")
        query_due_date = self.request.GET.get("due_date")
        query_start_date = self.request.GET.get("start_date")
        query_end_date = self.request.GET.get("end_date")
        query_status = self.request.GET.get("status")
        queryset = Invoice.objects.filter().search(query=query).filter_by_id(id=query_id).get_due_date(date=query_due_date).filter_date_range(start_date=query_start_date, end_date=query_end_date).get_status(status=query_status)
        return queryset


class RefundViewSet(viewsets.ModelViewSet):
    """
    This viewset automatically provides `list`, `create`, `retrieve`,
    `update` and `destroy` actions.

    Additionally we also provide an extra `highlight` action.
    """
    model = Refund
    serializer_class = RefundSerializer
    permission_classes = [permissions.IsAuthenticated,
                          permissions.IsAdminUser]

    def get_queryset(self):
        query = self.request.GET.get("q") 
        queryset = Refund.objects.filter().get_invoice(invoice_id=query)
        return queryset
    
class CreditViewSet(viewsets.ModelViewSet):
    """
    This viewset automatically provides `list`, `create`, `retrieve`,
    `update` and `destroy` actions.

    Additionally we also provide an extra `highlight` action.
    """
    model = Credit
    
    serializer_class = CreditSerializer
    permission_classes = [permissions.IsAuthenticated,
                          permissions.IsAdminUser]

    def get_queryset(self):
        query = self.request.GET.get("q") 
        queryset = Credit.objects.filter().get_customer_credit(customer_id=query)
        return queryset




import json
from django.urls.base import reverse, reverse_lazy
import sweetify
# import environ
from django.shortcuts import redirect, render
from django.http import HttpResponseRedirect
from django.utils.translation import gettext as _
# import time
from django.forms.models import modelform_factory

from tenant_users.tenants.utils import (get_tenant_model,
                                        get_tenant_domain_model)
from multitenancy.admin.decorators import allowed_users
from multitenancy.admin.models import Admin, Settings
from multitenancy.order.models import Order
from multitenancy.profiles.models import Profile
from tenant_users.tenants.models import InactiveError, ExistsError
from django.views.generic import View
from account.mixins import LoginRequiredMixin
from helpdesk.models import Ticket
from multitenancy.admin.filters import CustomerFilter


from multitenancy.subscriptions.models import Plan, ProductFeature, ProductType, Subscription
from pinax.teams.models import SimpleTeam, Team
from account.mixins import LoginRequiredMixin


from multitenancy.users.models import Customer, TenantUser
from multitenancy.apps.models import Tenant
from .baseViews import(
    AdminListView,
    AdminDeleteView,
    AdminCreateView,
    AdminUpdateView,
    AdminTemplateView,
    AdminView,
    AdminDetailView
    )
# env = environ.Env()

class AdminIndexView(LoginRequiredMixin ,AdminTemplateView):
    template_name = 'multitenancy/admin/adminUser/index.html'
    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)

        app_configs = Admin.app_configs
        # app_configs = sorted(app_configs, key=lambda x: x.menu_order)
        groups = {}
        context['apps'] = app_configs

        # for app_config in app_configs:
        #     group = groups.get(app_config.group, [])
        #     group.append(app_config)
        #     groups[app_config.group] = group
        #     context['apps'] = group

        context['subscriptions'] = get_tenant_model().objects.all().exclude(schema_name='public').exclude(is_template=True)
        context['users'] = TenantUser.objects.all()
        context['revenue'] = Tenant.total_revenue()
        ProductType.objects.create_defaults()
        context['staff'] = Profile.objects.filter()
        active_date = []
        active_count = []
        inactive_count = []
        for date in Subscription.get_active_inactive_subscriptions_data():
            active_date.append(date[0])
            active_count.append(date[1])
            inactive_count.append(date[2])

        context['active_date'] = json.dumps(active_date)
        context['active_count'] = json.dumps(active_count )
        context['inactive_count'] = json.dumps(inactive_count)
        
        context['customers'] = Customer.objects.filter().exclude(email="AnonymousUser")
        context['users'] = TenantUser.objects.all()
        context['active_tickets'] = Ticket.objects.select_related('queue').exclude(
        status__in=[Ticket.CLOSED_STATUS, Ticket.RESOLVED_STATUS],
        )
        return context


class TeamsIndexView(LoginRequiredMixin ,AdminTemplateView):
    template_name = "multitenancy/admin/adminUser/teamIndex.html"

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)

        context['simple_teams'] = SimpleTeam.objects.all()
        context['teams'] = Team.objects.all()
        return context






from rest_framework.test import APITestCase
from ..serializers import TenantSerializer
from multitenancy.subscriptions.models import Plan, Subscription, get_plans

class TenantSerializerTestCase(APITestCase):
    
        

    def test_serializer_with_valid_data(self):
        
        self.subscription = Subscription.objects.create()
        self.data = {
            'name': 'tenant_serialized',
            'type': 'personal',
            'is_template': False,
            # 'plan': 2,
            'subscription':{"product_type":{"name":"tenant"},"reason":"create subscription","reference":"99d8d","status":"active","cycle":"monthly", "subscription_duration":30},
            'slug':"tenant-serialized",
            'schema_name':"tenant-serialized",
            'description': 'Test serializer',
            'on_trail': True,
            'trail_duration': 30,
        }
        serializer = TenantSerializer(data=self.data)# type: ignore      
        
        self.assertTrue(serializer.is_valid(), msg=serializer.errors)

    


import unittest
from django.test import TestCase, Client
from multitenancy.apps.models import Tenant, Domain
from multitenancy.subscriptions.models import Subscription
from multitenancy.users.models import Admin, Customer, TenantUser
from tenant_users.permissions.models import UserTenantPermissions
from multitenancy.utils import create_public_tenant

class TestTenant(unittest.TestCase):
    def  setUp(self) -> None:
        self.client = Client()
        
    
    def test_start_trail(self):
        public_user = TenantUser.objects.create(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='public@email.com', 
            type='Admin',
            is_active=True
            )
        #Create public tenant
        
        public_tenant = create_public_tenant("localhost", "tkhambane@gmail.com", "publicuser123")
        
        # Create a tenant
        user = TenantUser.objects.create_user(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abcpp@email.com', 
            type='Customer',
            is_active=True
            )
        subscription = Subscription.objects.create()
        tenant = Tenant.objects.create(name="Test Tenant", type="personal", is_template=False, description="Test tenant for testing purposes", owner=user, schema_name='tenant1', subscription=subscription)
        domain = Domain.objects.create(domain="domain1.com", tenant=tenant, is_primary=True)
        tenant.add_user(user, is_superuser=True, is_staff=True)
        tenant.auto_create_schema = False
        tenant.save()
        
        self.assertFalse(tenant.subscription.is_active)
        self.assertEqual(tenant.name, "Test Tenant")
        self.assertEqual(tenant.type, "personal")
        self.assertFalse(tenant.is_template)
        self.assertEqual(tenant.description, "Test tenant for testing purposes")
        self.assertFalse(tenant.on_trial)

        # Start the trail for the tenant
        tenant.start_trail()
        self.assertTrue(tenant.subscription.is_active)
        self.assertTrue(tenant.on_trial)
        self.assertIsNotNone(tenant.trail_duration)

        # Try to start the trail again
        tenant.start_trail()
        self.assertTrue(tenant.on_trial)
        self.assertEquals(30,tenant.trail_duration)


    def test_end_trail(self):
        
        user = TenantUser.objects.create(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc2@email.com', 
            type='Customer',
            is_active=True
            )
        subscription = Subscription.objects.create()
        tenant = Tenant.objects.create(name="Test Tenant", type="personal", is_template=False,
                                        description="Test tenant for testing purposes",
                                          owner=user, schema_name='tenant2', subscription=subscription)
        domain = Domain.objects.create(domain="domain2.com", tenant=tenant, is_primary=True)
        tenant.add_user(user, is_superuser=True, is_staff=True)
        tenant.auto_create_schema = False
        tenant.save()
        # End the trail
        tenant.end_trail()
        self.assertFalse(tenant.on_trial)
        self.assertEqual(0,tenant.trail_duration)
        
    def test_trail_days_start(self):
        user = TenantUser.objects.create(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc3@email.com', 
            type='Customer',
            is_active=True
            )
        subscription = Subscription.objects.create()
        tenant = Tenant.objects.create(name="Test Tenant", type="personal", is_template=False,
                                        description="Test tenant for testing purposes", 
                                        owner=user, schema_name='tenant3', on_trial=True, 
                                        trail_duration=30, subscription=subscription)
        domain = Domain.objects.create(domain="domain3.com", tenant=tenant, is_primary=True)
        tenant.add_user(user, is_superuser=True, is_staff=True)
        tenant.auto_create_schema = False
        tenant.save()
        # Check trail days
        tenant.trail_days_left()
        self.assertTrue(tenant.on_trial)
        self.assertEqual(30,tenant.trail_duration)
        

    def test_trail_days_end(self):
        user = TenantUser.objects.create(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc4@email.com', 
            type='Customer',
            is_active=True
            )
        subscription = Subscription.objects.create()
        tenant = Tenant.objects.create(name="Test Tenant", type="personal", 
                                       is_template=False, description="Test tenant for testing purposes", 
                                       owner=user, schema_name='tenant4', 
                                       on_trial=True, trail_duration=30, subscription=subscription)
        domain = Domain.objects.create(domain="domain4.com", tenant=tenant, is_primary=True)
        tenant.add_user(user, is_superuser=True, is_staff=True)
        tenant.auto_create_schema = False
        tenant.save()
        tenant.trail_duration = 0
        tenant.save()
        # Check trail days
        tenant.trail_days_left()
        self.assertFalse(tenant.on_trial)
        self.assertEqual(0,tenant.trail_duration)
    
    


# Generated by Django 3.2.12 on 2023-02-01 18:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0012_alter_subscription_subscription_duration'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='plan',
            options={'ordering': ('-price',)},
        ),
        migrations.AlterModelOptions(
            name='productfeature',
            options={'ordering': ('-pk',)},
        ),
        migrations.AlterModelOptions(
            name='subscription',
            options={'ordering': ('-last_updated',), 'verbose_name': 'Subscription'},
        ),
    ]



from django.urls import path

from . import views

urlpatterns = [
    path('billing/orders/',views.OrdersListView.as_view(), name="orders_list"),
    path('billing/orders/dashboard/', views.OrderIndexView.as_view(), name="orders_index" ),
]


from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend


class EmailBackEnd(ModelBackend):
    def authenticate(self, username=None, password=None, **kwargs):
        UserModel = get_user_model()
        try:
            user = UserModel.objects.get(email=username)
        except UserModel.DoesNotExist:
            return None
        else:
            if user.check_password(password):
                return user
        return None



# Generated by Django 3.2.12 on 2023-01-01 04:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0004_auto_20221113_1106'),
    ]

    operations = [
        migrations.AlterField(
            model_name='plan',
            name='name',
            field=models.CharField(choices=[('personal', 'personal'), ('premium', 'premium'), ('business', 'business')], max_length=250, unique=True),
        ),
    ]



#!/usr/bin/env python
import fileinput
import fnmatch
import os
import re
import sys
from argparse import ArgumentParser
from difflib import unified_diff

from django.core.management import ManagementUtility

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 7)

if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write(
        "This version of dajgno-Multitenacy requires Python {}.{} or above - you are running {}.{}\n".format(
            *(REQUIRED_PYTHON + CURRENT_PYTHON)
        )
    )
    sys.exit(1)


def pluralize(value, arg="s"):
    return "" if value == 1 else arg


class Command:
    description = None

    def create_parser(self, command_name=None):
        if command_name is None:
            prog = None
        else:
            # hack the prog name as reported to ArgumentParser to include the command
            prog = "%s %s" % (prog_name(), command_name)

        parser = ArgumentParser(
            description=getattr(self, "description", None), add_help=False, prog=prog
        )
        self.add_arguments(parser)
        return parser

    def add_arguments(self, parser):
        pass

    def print_help(self, command_name):
        parser = self.create_parser(command_name=command_name)
        parser.print_help()

    def execute(self, argv):
        parser = self.create_parser()
        options = parser.parse_args(sys.argv[2:])
        options_dict = vars(options)
        self.run(**options_dict)


class CreateProject(Command):
    description = "Creates the directory structure for a new Multitenancy project."

    def add_arguments(self, parser):
        parser.add_argument("project_name", help="Name for your Multitenancy project")
        parser.add_argument(
            "dest_dir",
            nargs="?",
            help="Destination directory inside which to create the project",
        )
        parser.add_argument(
            '--sitename',
            help='Human readable name of your organisation or brand, e.g. "Mega Corp Inc."'
        )
        parser.add_argument(
            '--domain',
            help='Domain that will be used for your website in production, e.g. "www.example.com"'
        )
        parser.add_argument(
            '--database',
            help='The database name you will be using to connect your project.'
        )
        parser.add_argument(
            '--password',
            help='The password for your database'
        )
        parser.add_argument(
            '--port',
            help='The port for your database'
        )

    def run(self, project_name=None, dest_dir=None, sitename="", domain="", database="", password="", port=""):
        # Make sure given name is not already in use by another python package/module.
        try:
            __import__(project_name)
        except ImportError:
            pass
        else:
            sys.exit(
                "'%s' conflicts with the name of an existing "
                "Python module and cannot be used as a project "
                "name. Please try another name." % project_name
            )

        print(  # noqa
            "Creating a Multitenancy project called %(project_name)s"
            % {"project_name": project_name}
        )  # noqa

        # Create the project from the Wagtail template using startapp

        # First find the path to Wagtail
        import multitenancy

        multitenancy_path = os.path.dirname(multitenancy.__file__)
        template_path = os.path.join(multitenancy_path, "project_template")

        # Call django-admin startproject
        utility_args = [
            "django-admin",
            "startproject",
            "--template=" + template_path,
            "--ext=html,rst,txt,md,py",
            "--name=Dockerfile",
            "--sitename",
            project_name,
            sitename,
            domain,
            database,
            password,
            port
        ]

        if dest_dir:
            utility_args.append(dest_dir)

        utility = ManagementUtility(utility_args)
        utility.execute()

        print(  # noqa
            "Success! %(project_name)s has been created"
            % {"project_name": project_name}
        )  # noqa








COMMANDS = {
    "start": CreateProject(),
    
}


def prog_name():
    return os.path.basename(sys.argv[0])


def help_index():
    print(  # noqa
        "Type '%s help <subcommand>' for help on a specific subcommand.\n" % prog_name()
    )  # NOQA
    print("Available subcommands:\n")  # NOQA
    for name, cmd in sorted(COMMANDS.items()):
        print("    %s%s" % (name.ljust(20), cmd.description))  # NOQA


def unknown_command(command):
    print("Unknown command: '%s'" % command)  # NOQA
    print("Type '%s help' for usage." % prog_name())  # NOQA
    sys.exit(1)


def main():
    try:
        command_name = sys.argv[1]
    except IndexError:
        help_index()
        return

    if command_name == "help":
        try:
            help_command_name = sys.argv[2]
        except IndexError:
            help_index()
            return

        try:
            command = COMMANDS[help_command_name]
        except KeyError:
            unknown_command(help_command_name)
            return

        command.print_help(help_command_name)
        return

    try:
        command = COMMANDS[command_name]
    except KeyError:
        unknown_command(command_name)
        return

    command.execute(sys.argv)


if __name__ == "__main__":
    main()



from django.views.generic import TemplateView, ListView, DeleteView, View, CreateView, UpdateView, DetailView
from django.contrib.auth.mixins import UserPassesTestMixin
from django.template.loader import get_template
from django.http import Http404, HttpResponseNotFound
from django.utils.decorators import method_decorator
from django.utils.translation import gettext as _
from multitenancy.admin.decorators import allowed_users





class AdminView(UserPassesTestMixin, View):
    redirect_field_name = 'next'
    def test_func(self):
        return self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)

class AdminCreateView(UserPassesTestMixin, CreateView):
    redirect_field_name = 'next'
    def test_func(self):
        return self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)

class AdminDetailView(UserPassesTestMixin, DetailView):
    redirect_field_name = 'next'
    def test_func(self):
        return self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)

    


class AdminUpdateView(UserPassesTestMixin, UpdateView):
    redirect_field_name = 'next'
    def test_func(self):
        return self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)
    
    def get(self, request, *args, **kwargs):
        self.object = None
        return super().get(request, *args, **kwargs)



class AdminDeleteView(UserPassesTestMixin, DeleteView):
    redirect_field_name = 'next'
    def test_func(self):
        return self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)


class AdminListView(UserPassesTestMixin, ListView):
    redirect_field_name = 'next'
    def test_func(self):
        return self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)
    


class AdminTemplateView(UserPassesTestMixin, TemplateView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)

class StaffCreateView(UserPassesTestMixin, CreateView):
    redirect_field_name = 'next'
    def test_func(self):
        return self.request.user.type == 'Staff'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)


class StaffUpdateView(UserPassesTestMixin, UpdateView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Staff'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)


class StaffDeleteView(UserPassesTestMixin, DeleteView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Staff'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)


class StaffListView(UserPassesTestMixin, ListView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Staff'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)

class StaffTemplateView(UserPassesTestMixin, TemplateView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Staff'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)


class TeamDetailView(UserPassesTestMixin, DetailView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Staff' or self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)


class TeamCreateView(UserPassesTestMixin, CreateView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Staff' or self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)


class TeamUpdateView(UserPassesTestMixin, UpdateView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Staff' or self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)


class TeamDeleteView(UserPassesTestMixin, DeleteView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Staff' or self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)


class TeamListView(UserPassesTestMixin, ListView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Staff' or self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)

class TeamTemplateView(UserPassesTestMixin, TemplateView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Staff' or self.request.user.type == 'Admin'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)
    

class CustomerView(UserPassesTestMixin, View):
    redirect_field_name = 'next'
    def test_func(self):
        return self.request.user.type == 'Customer'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)

class CustomerTemplateView(UserPassesTestMixin, TemplateView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Customer'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)
    
class CustomerListView(UserPassesTestMixin, ListView):
    redirect_field_name = 'next'
    def test_func(self):
        return  self.request.user.type == 'Customer'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)

class CustomerDetailView(UserPassesTestMixin, DetailView):
    redirect_field_name = 'next'
    def test_func(self):
        return self.request.user.type == 'Customer'
    
    def handle_no_permission(self):
        template = get_template('404.html')
        html = template.render()
        return HttpResponseNotFound(html)



# Generated by Django 3.2.12 on 2023-01-08 13:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0008_auto_20230108_1539'),
    ]

    operations = [
        migrations.AlterField(
            model_name='plan',
            name='slug',
            field=models.SlugField(),
        ),
    ]



from django.urls import path, include
from rest_framework.routers import DefaultRouter
from multitenancy.subscriptions import views as subscriptions_views
from multitenancy.apps import views as tenant_views
from multitenancy.users import views as users_views
from multitenancy.billing import views as billing_views

router = DefaultRouter()
router.register(r'customers', users_views.CustomerViewSet,basename="customers")
router.register(r'staff', users_views.StaffViewSet,basename="staff")
router.register(r'plans', subscriptions_views.PlanViewSet,basename="plans", )
router.register(r'subscriptions', subscriptions_views.SubscriptionsViewSet,basename="subscriptions")
router.register(r'billing/invoices', billing_views.InvoiceViewSet,basename="invoices")
router.register(r'billing/refunds', billing_views.RefundViewSet,basename="refunds")
router.register(r'billing/credits', billing_views.CreditViewSet,basename="credits")
router.register(r'templates', tenant_views.TenantTemplateViewSet,basename="templates")

urlpatterns = [
    path('', include(router.urls)),
]


# Generated by Django 3.2.12 on 2022-12-10 13:24

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0004_auto_20221113_1106'),
        ('apps', '0009_domain_has_custom'),
    ]

    operations = [
        migrations.DeleteModel(
            name='TenantType',
        ),
        migrations.RemoveField(
            model_name='tenant',
            name='subscription',
        ),
        migrations.AlterField(
            model_name='tenant',
            name='on_trial',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='tenant',
            name='plan',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='subscriptions.plan'),
        ),
        migrations.DeleteModel(
            name='Package',
        ),
    ]



from django.urls import path
from . import views

urlpatterns = [
    path("customers/create/", views.CreateCustomerView.as_view(), name="create_customer"),
    path("customers/<int:pk>/update/", views.UpdateCustomerView.as_view(), name="update_customer"),
    path(route="customers/", view=views.CustomerListView.as_view(), name="customer_list", ),
    path("customers/<int:pk>/delete/", views.DeleteCustomerView.as_view(), name="delete_customer"),
    path(route="staff/", view=views.StaffListView.as_view(), name="staff_list", ),
    path("staff/<int:pk>/delete/", views.DeleteStaffView.as_view(), name="delete_staff"),
    path("staff/create/", views.CreateStaffView.as_view(), name="create_staff"),
    path("staff/<int:pk>/update/", views.UpdateStaffView.as_view(), name="update_staff"),



]


from multitenancy.apps.views import TenantViewSet, api_root
from rest_framework import renderers

tenant_list = TenantViewSet.as_view({
    'get': 'list',
    'post': 'create'
})
tenant_detail = TenantViewSet.as_view({
    'get': 'retrieve',
    'put': 'update',
    'patch': 'partial_update',
    'delete': 'destroy'
})



# Generated by Django 3.2.12 on 2022-11-21 15:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0005_auto_20221116_1150'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='adminsettings',
            options={'verbose_name': 'Time zone and currency'},
        ),
    ]



import os
import uuid
from django.db import models
from django.utils import timezone
from django.conf import settings


def avatar_upload(instance, filename):
    ext = filename.split(".")[-1]
    filename = "%s.%s" % (uuid.uuid4(), ext)
    return os.path.join("avatars", filename)


class Skills(models.Model):
    name = models.CharField(max_length=150, blank=False, null=False)

    def __str__(self) -> str:
        return self.name


class Profile(models.Model):

    user = models.OneToOneField(settings.AUTH_USER_MODEL, related_name="profile", on_delete=models.PROTECT)  # type: ignore
    name = models.CharField(max_length=75, blank=True)
    job_title = models.CharField(max_length=75, blank=True)
    avatar = models.ImageField(upload_to=avatar_upload, blank=True)
    bio = models.TextField(blank=True)
    skills = models.ManyToManyField(Skills, blank=True)
    education = models.CharField(max_length=100, blank=True)
    display_profile = models.BooleanField(default=True)
    location = models.CharField(max_length=100, blank=True)
    website = models.URLField(max_length=250, blank=True)
    twitter_username = models.CharField("Twitter Username", max_length=100, blank=True)

    created_at = models.DateTimeField(default=timezone.now)
    modified_at = models.DateTimeField(default=timezone.now)

    def save(self, *args, **kwargs):
        self.modified_at = timezone.now()
        return super(Profile, self).save(*args, **kwargs)

    @property
    def display_name(self):
        return self.user
        # else:
        #     return self.user.username



# Generated by Django 3.2.12 on 2023-02-09 14:35

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('billing', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PaymentGateWay',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=250)),
                ('description', models.TextField()),
                ('status', models.CharField(choices=[('active', 'Active'), ('inactive', 'Inactive')], max_length=10)),
                ('default', models.BooleanField(default=False)),
            ],
        ),
        migrations.AlterModelOptions(
            name='invoice',
            options={'ordering': ['-date_created']},
        ),
        migrations.CreateModel(
            name='Payment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=10)),
                ('status', models.CharField(choices=[('success', 'Success'), ('failed', 'Failed'), ('pending', 'Pending')], max_length=10)),
                ('gateway', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='billing.paymentgateway')),
                ('subscriber', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 3.2.12 on 2023-01-24 17:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0006_alter_adminsettings_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='logo',
            name='logo',
            field=models.ImageField(default='core/logo.png', upload_to=''),
        ),
    ]



from  rest_framework import serializers
from .models import Staff, Customer, Admin


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = Staff
        fields = '__all__'
        # fields = ['id', 'first_name', 'last_name', 'username', 'email', 'type','signup_confirmation', 'avatar', 'last_login']



# Generated by Django 3.2.12 on 2022-11-14 06:17

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Logo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('logo', models.ImageField(upload_to='')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# from django.test import TestCase

# from django.urls import reverse, resolve
# from multitenancy.profiles.views import ProfileListView, ProfileDetailView, ProfileEditView

# class TestUrls(TestCase):

#     def test_list_resolved(self):
#         url = reverse('profiles_list')
#         self.assertEquals(resolve(url).func, ProfileListView)



from django.contrib import admin
from multitenancy.users.models import TenantUser

@admin.register(TenantUser)
class TenantUserAdmin(admin.ModelAdmin):
    list_display = ('id', 'username', 'first_name', 'last_name', 'email', 'type')

admin.site.site_url = "/admin/"



from django.shortcuts import redirect, render
import functools
from django.contrib import messages
from django.conf import settings


def allowed_users(allowed_types=[]):
    def decorator(view_func):
        def wrapper(self, *args, **kwargs):
            if self.request.user.is_authenticated:
                if self.request.user.type in allowed_types:
                    return view_func(self, *args, **kwargs)
                elif self.request.user.type == "Admin":
                    return render(self.request, '404.html')
                elif self.request.user.type == "Staff":
                    return render(self.request, '404.html')
                elif self.request.user.type == "Customer":
                    return render(self.request, '404.html')
            else:
                return redirect("accounts_login")
        return wrapper
    return decorator

# def allowed_users(allowed_types=[]):
#     def authenticated_user(view_func):
#         def wrapper_func(request, *args, **kwargs):
#             if request.user.is_authenticated:
#                 if request.user.type in allowed_types:
#                     return view_func(request, *args, **kwargs)
#                 if request.user.type == "Admin":
#                     return render(request, '404.html')
#                 elif request.user.type == "Staff":
#                     return render(request, '404.html')
#                 elif request.user.type == "Customer":
#                     return render(request, '404.html')
#             else:
#                 return redirect(settings.LOGIN_REDIRECT_URL)
#         return wrapper_func
#     return authenticated_user

def verification_required(view_func, verification_url="accounts:activate_email"):
    """
        this decorator restricts users who have not been verified
        from accessing the view function passed as it argument and
        redirect the user to page where their account can be activated
    """
    @functools.wraps(view_func)
    def wrapper(request, *args, **kwargs):
        if request.user.is_active:
            return view_func(request, *args, **kwargs)
        messages.info(request, "Email verification required")
        print("You need to be logged out")
        return redirect(verification_url)  
    return wrapper



# Generated by Django 3.2.12 on 2023-02-13 17:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0009_alter_generalinfo_website'),
    ]

    operations = [
        migrations.CreateModel(
            name='Currency',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=3)),
                ('name', models.CharField(max_length=50)),
                ('symbol', models.CharField(max_length=10)),
                ('default', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""
Default settings for Multitenancy apps
"""

from django.conf import settings

#The default name for your tenants, eg if your saas app is a website you can name it website
#or CRM you can name it CRM etc.



TENANT_DISPLAY_NAME = getattr(settings, "TENANT_DISPLAY_NAME", "Service")
TENANT_DISPLAY_NAME_PLURAL =getattr(settings, "TENANT_DISPLAY_NAME_PLURAL", "Services")
TENANT_CREATION_TEMPLATE = getattr(settings, "TENANT_CREATION_TEMPLATE", "multitenancy/admin/publicUser/create_service.html")


from django.test import TestCase
import json
import datetime
from ..models import Plan, ProductFeature,ProductType, Subscription
from ..serializers import PlanSerialiser, ProductFeatureSerializer, ProductTypeSerializer, SubscriptionSerializer

import json

class PlanSerializerTestCase(TestCase):

    def setUp(self):
        self.product_feature_1 = ProductFeature.objects.create(
            name='Feature 1'
        )
        self.product_feature_2 = ProductFeature.objects.create(
            name='Feature 2'
        )
        self.plan = Plan.objects.create(
            name='Plan 1',
            description='Plan 1 description',
            price=100.0,
        )
        self.plan.features.add(self.product_feature_1, self.product_feature_2)

    def test_plan_serializer(self):
        #plan = Plan.objects.get(id=self.plan.id)
        serialized_data = PlanSerialiser(self.plan).data
        deserialized_data = json.loads(json.dumps(serialized_data))
        self.assertEqual(self.plan.pk, deserialized_data['id'])
        self.assertEqual(self.plan.name, deserialized_data['name'])
        self.assertEqual(self.plan.description, deserialized_data['description'])
        self.assertEqual(self.plan.price, float(deserialized_data['price']))
        # Convert QuerySet to list of dictionaries
        actual_fueatures_data = list(self.plan.features.all().values('id', 'name'))
        self.assertEqual(actual_fueatures_data, deserialized_data['features'])
        # for feature in self.plan.features.all():
        #     self.assertEqual(feature.id, serialized_data)
            

class ProductFeatureSerializerTestCase(TestCase):

    def setUp(self):
        self.product_feature = ProductFeature.objects.create(
            name='Feature 1'
        )

    def test_product_feature_serializer(self):
        feature = ProductFeature.objects.get(id=self.product_feature.pk)
        serializer = ProductFeatureSerializer(feature)
        serialized_data = json.dumps(serializer.data)
        self.assertIn('"id": {}'.format(feature.pk), serialized_data)
        self.assertIn('"name": "{}"'.format(feature.name), serialized_data)


class ProductTypeSerializerTestCase(TestCase):

    def setUp(self):
        self.product_type = ProductType.objects.create(
            name='Type 1'
        )

    def test_product_type_serializer(self):
        type = ProductType.objects.get(id=self.product_type.id)
        serializer = ProductTypeSerializer(type)
        serialized_data = json.dumps(serializer.data)
        self.assertIn('"id": {}'.format(type.id), serialized_data)
        self.assertIn('"name": "{}"'.format(type.name), serialized_data)

class SubscriptionSerializerTestCase(TestCase):

    def setUp(self):
        self.product_type = ProductType.objects.create(
            name='Type 1'
        )
        self.subscription = Subscription.objects.create(
            cycle="monthly",
            subscription_duration=1,
            end_date=datetime.date.today() + datetime.timedelta(days=30),
            renewal_date=datetime.date.today() + datetime.timedelta(days=30),
            reference='123456789',
            last_updated=datetime.datetime.now(),
            product_type=self.product_type,
            reason='Test reason',
            status="active",
        )

    def test_subscription_serializer(self):
        subscription = Subscription.objects.get(id=self.subscription.id)
        #serializer = SubscriptionSerializer(subscription)
        # serialized_data = json.dumps(serializer.data)
        serialized_data = SubscriptionSerializer(self.subscription).data
        deserialized_data = json.loads(json.dumps(serialized_data))
        start_date = self.subscription.start_date
        end_date = self.subscription.end_date
        self.assertEqual(self.subscription.id, deserialized_data['id'])
        self.assertEqual(self.subscription.cycle, deserialized_data['cycle'])
        self.assertEqual(self.subscription.subscription_duration, deserialized_data['subscription_duration'])
        self.assertEqual(start_date.strftime('%Y-%m-%d')  , deserialized_data['start_date'])
        self.assertEqual(end_date.strftime('%Y-%m-%d'), deserialized_data['end_date'])
        self.assertEqual(self.subscription.created_date.strftime('%Y-%m-%d'), deserialized_data['created_date'])
        self.assertEqual(self.subscription.renewal_date.strftime('%Y-%m-%d'), deserialized_data['renewal_date'])
        


import datetime
from django.db import models
from django.db.models import Q
from calendar import monthrange
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.utils.text import slugify 
from multitenancy.users.models import Customer
from django_tenants.utils import get_tenant_type_choices


def get_plans():
    #remove the plublic tenant type from list
    items = get_tenant_type_choices()
    items.pop(0) 
    return items

class ProductFeature(models.Model):
    name = models.CharField(max_length=250)
    description = models.TextField()

    def __str__(self) -> str:
        return self.name
    
    class Meta:
        ordering= ('-pk',)


class PlanQueryset(models.QuerySet):

    def search(self, query):
        return self.filter(
            Q(name__icontains=query) |
            Q(description__icontains=query)
        )

    def active_plans(self):
        return self.filter(is_active=True)

    def by_price(self, price):
        return self.filter(price=price)

    def by_features(self, feature):
        return self.filter(features=feature)

class PlanManager(models.Manager):
    def get_queryset(self):
        return PlanQueryset(self.model, using=self._db)

    def search(self, query):
        return self.get_queryset().search(query)

    def active_plans(self):
        return self.get_queryset().active_plans()

    def by_price(self, price):
        return self.get_queryset().by_price(price)

    def by_features(self, feature):
        return self.get_queryset().by_features(feature)

class Plan(models.Model):
    name = models.CharField(max_length=250, blank=False, unique=True, null=False, choices=get_plans())
    slug = models.SlugField()
    description = models.TextField(blank=True, null=True)
    features = models.ManyToManyField(ProductFeature)
    price = models.DecimalField(default=75,  # type: ignore
                                max_digits=12, verbose_name="Price", decimal_places=2)
    objects = PlanManager()
    
    class Meta:
        ordering = ('-price',)

    def __str__(self):
        return self.name
    
    def add_feature(self, feature):
        new_feature = ProductFeature.objects.create(name=feature)
        self.features.add(new_feature)
        self.save()

    def save(self, *args, **kwargs):  # new
        if not self.slug:
            self.slug = slugify(self.name)
        return super().save(*args, **kwargs)
    
    @property
    def price_weekly(self):
        price = self.price / 4
        return price
    
    @property
    def price_quartely(self):
        price = self.price * 3
        return price

    @property
    def price_annually(self):
        price = self.price * 12
        return price


class ProductTypeManager(models.Manager):
    def create_defaults(self):
        self.get_or_create(name=ProductType.Types.TENANT_APP)
        self.get_or_create(name=ProductType.Types.DOMAIN)
        self.get_or_create(name=ProductType.Types.THIRD_PARTY_APP)

class ProductType(models.Model):
    class Types(models.TextChoices):
        TENANT_APP = "tenant", "Tenant App"
        DOMAIN = "domain", "Custom Domain"
        THIRD_PARTY_APP = "third_party", "3rd party App"
    name = models.CharField(max_length=115,choices=Types.choices, default=Types.TENANT_APP)
    objects = ProductTypeManager()


class SubscriptionQueryset(models.QuerySet):
    def active(self):
        return self.filter(status="active")

    def inactive(self):
        return self.filter(status="inactive")

    def cancelled(self):
        return self.filter(status="cancelled")

    def expired(self):
        return self.filter(status="expired")

    def weekly(self):
        return self.filter(cycle=Subscription.Cycles.WEEKLY)

    def monthly(self):
        return self.filter(cycle=Subscription.Cycles.MONTHLY)

    def quarterly(self):
        return self.filter(cycle=Subscription.Cycles.QUARTERLY)

    def annually(self):
        return self.filter(cycle=Subscription.Cycles.ANNUALLY)

    def started_within_week(self):
        return self.filter(start_date__gte=timezone.now() - timezone.timedelta(days=7))

    def ended_within_week(self):
        return self.filter(end_date__lte=timezone.now() - timezone.timedelta(days=7))

    def renew_within_week(self):
        return self.filter(renewal_date__lte=timezone.now() - timezone.timedelta(days=7))
    def get_product_type(self, name):
        return self.filter(product_type__name__exact=name)

    def get_status(self, query):
        return self.filter(status__exact=query)

    def get_active(self):
        return self.filter(status__contains="active")
    
    def search(self, query=None):
        if query is None or query == "":
            return self.all()
        return self.filter(
            Q(reference__icontains=query) |
            Q(reason__icontains=query) |
            Q(product_type__name__icontains=query)
        )

    


class Subscription(models.Model):
    class Cycles(models.TextChoices):
        WEEKLY = "weekly", "Weekly"
        MONTHLY = "monthly", "Monthly"
        QUARTERLY = "quartely", "quarterly"
        ANNUALLY = "annually", "annually"
    
    cycle = models.CharField(max_length=50, choices=Cycles.choices, default=Cycles.MONTHLY)
    subscription_duration = models.IntegerField(default=0)
    start_date = models.DateField(auto_now_add=True)
    created_date = models.DateField(auto_now_add=True,null=True)
    end_date = models.DateField(auto_now_add=True)
    renewal_date = models.DateField(null=True)
    reference = models.TextField(max_length=100, help_text="Free text field for user references")
    last_updated = models.DateTimeField(
        auto_now=True, help_text="Keeps track of when a record was last updated"
    )
    product_type = models.ForeignKey(ProductType,null=True, on_delete=models.CASCADE, related_name="subscriptions")
    reason = models.TextField(help_text="Reason for state change, if applicable.")
    status = models.CharField(max_length=10, choices=[('active', 'Active'),('inactive', 'Inactive') ,('cancelled', 'Cancelled'), ('expired', 'Expired')], default="inactive")
    objects = SubscriptionQueryset().as_manager()


    def __str__(self) -> str:
        return f"{self.pk}"

    
    class Meta:
        verbose_name = _("Subscription")
        ordering = ("-last_updated",)

    
    @classmethod
    def get_active_inactive_subscriptions_data(cls):
        """
        Create a dataset for total active and inactive subscriptions, the data is updated daily and the data can be displayed on a dashboard graph
        """
        today = timezone.now().date()
        first_day_of_month = today.replace(day=1)
        last_day_of_month = today.replace(day=monthrange(today.year, today.month)[1])
        data = []
        for i in range((last_day_of_month - first_day_of_month).days + 1):
            date = first_day_of_month + datetime.timedelta(days=i)
            if date <= today:
                active_count = cls.objects.filter(status="active", created_date__lte=date).count()
                inactive_count = cls.objects.filter(status="inactive", created_date__lte=date).count()
                
            else:
                active_count = 0
                inactive_count = 0
            data.append((date.strftime("%Y-%m-%d"), active_count, inactive_count))
            # active_data[date.strftime("%Y-%m-%d")] = active_count
            # inactive_data[date.strftime("%Y-%m-%d")] = inactive_count
        return data

    def start_subscription(self, cycle):
        
        # calculate the subscription_duration based on the subscription cycle
        self.status = "active"
        self.cycle = cycle
        self.reason =  "Start Subscription"
        self.renewal_date = datetime.date.today()
        self.save()
        if cycle == 'weekly':
            self.subscription_duration =  7
            self.end_date =  self.renewal_date + datetime.timedelta(days=self.subscription_duration)
            self.save()
            return self.subscription_duration
        elif  cycle == 'monthly':
            self.subscription_duration = 30
            self.end_date = self.renewal_date + datetime.timedelta(days=self.subscription_duration)
            self.save()
            return self.subscription_duration
        elif  cycle == 'quartely':
            self.subscription_duration = 90
            self.end_date = self.renewal_date + datetime.timedelta(days=self.subscription_duration)
            self.save()
            return self.subscription_duration
        else:
            self.subscription_duration = 365
            self.end_date = self.renewal_date + datetime.timedelta(days=self.subscription_duration)
            self.save()
            return self.subscription_duration

    def renew(self):
        
        if self.status == 'active':
            # Calculate and update the new end date for the subscription
            self.end_date = self.end_date + datetime.timedelta(days=self.subscription_duration)
            # Set the renewal date to today
            self.renewal_date = datetime.date.today()
            # Update the reason 
            self.reason = "Subscription renewed"
            self.save()
            # Charge the customer here and create invoice
        elif self.status == 'expired':
            # Send a notification to the customer here
            pass
    
    def cancel_subscription(self):
        # set the status to cancelled
        if self.status == 'active':
            self.status = 'cancelled'
            # Set the end date to today
            self.end_date = datetime.date.today()
            # Update the reason 
            self.reason = "Subscription cancelled"
            self.save()

    def activate_subscription(self, duration):
        # In future checks will be done to ensure there are no outstanding invoices
        # update the renewal status
        self.status = 'active'
        # Set the renewal date to today
        self.renewal_date = datetime.date.today()
        # Calculate and update the new end date for the subscription
        self.end_date = self.renewal_date + datetime.timedelta(days=duration)
        # Update the reason 
        self.reason = "Subscription activated"
        self.save()

    def update_duration(self, duration):
        self.subscription_duration = duration
        self.save()



    @property
    def is_active(self):
        # a check to see if the active
        if self.status == "active":
            return True
        else:
            return False

    def get_service(self):
        pass

    def get_product_type(self, type:str):
        # On service creation each subscription must be assigned to a product type
        self.product_type = type
        self.save()



from django.urls.base import reverse

from multitenancy.admin.views.baseViews import TeamUpdateView, TeamListView, TeamDetailView

from django.contrib import messages

from account.mixins import LoginRequiredMixin

from .forms import ProfileForm
from .models import Profile


class ProfileEditView(LoginRequiredMixin, TeamUpdateView):

    form_class = ProfileForm
    model = Profile

    def get_object(self):
        return self.request.user.profile

    def get_success_url(self):
        return reverse("profiles_list")

    def form_valid(self, form):
        response = super(ProfileEditView, self).form_valid(form)
        messages.success(self.request, "You successfully updated your profile.")
        return response


class ProfileDetailView(LoginRequiredMixin, TeamDetailView):

    model = Profile
    slug_url_kwarg = "username"
    slug_field = "user__username"
    context_object_name = "profile"


class ProfileListView(LoginRequiredMixin, TeamListView):

    model = Profile
    context_object_name = "profiles"



# Generated by Django 3.2.12 on 2023-02-04 09:52

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('recipient_name', models.CharField(max_length=100)),
                ('street_address', models.CharField(max_length=100)),
                ('city', models.CharField(max_length=50)),
                ('state', models.CharField(max_length=50)),
                ('postal_code', models.CharField(max_length=20)),
                ('country', models.CharField(max_length=50)),
            ],
        ),
    ]



# Generated by Django 3.2.12 on 2022-11-12 11:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0001_initial'),
        ('apps', '0006_alter_tenant_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='tenant',
            name='subscription',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='subscriptions.subscription'),
        ),
    ]



import unittest
from django.test import RequestFactory, TestCase, Client
from django.urls import reverse, reverse_lazy
from tenant_users.permissions.models import UserTenantPermissions
from multitenancy.admin.views.adminViews import (
    AdminIndexView, 
    )
from multitenancy.apps.models import Tenant

from multitenancy.apps.views import (
    TenantListView, 
    UpdateTemplateView,
    CustomerTenantListView,
    CreateTemplateView,
    TemplateListView,
    DeleteTemplateView
    )
from multitenancy.subscriptions.models import Subscription
from multitenancy.subscriptions.views import (
    CreatePlanView, 
    PlanDetailView,
    PlanListView,
)
from multitenancy.users.models import TenantUser


class TenantTemplateViewsTestCase(unittest.TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.client = Client()
        self.tenant_data = {
            'name': 'tenanttest1',
            'slug': 'tenanttest1',
            'type': 'premium',
        }
        self.new_tenant_data = {
            'type': 'business',
        }
        
    def test__templatelist_view(self):
        self.user = TenantUser.objects.create_superuser(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc12fff3@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/templates/')
        request.user = self.user
        response = TemplateListView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(self.client.session.get('_auth_user_id'))
        self.assertIn('_auth_user_id', self.client.session)

    def test_get_create_template_view(self):
        self.user = TenantUser.objects.create_superuser(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc1ppp23@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/templates/create/')
        request.user = self.user
        response = CreateTemplateView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(self.client.session.get('_auth_user_id'))
        self.assertIn('_auth_user_id', self.client.session)

    
    def test_post_create_temaple_view(self):
        self.user = TenantUser.objects.create_superuser(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc1pp23@email.com', 
            type='Admin',
            is_active=True
            )
        self.client.force_login(user=self.user)
        request = self.factory.post('/admin/template/create/', data=self.tenant_data)
        request.user = self.user
        request.session = {}
        response = CreateTemplateView.as_view()(request)
        print(response)
        self.assertEqual(response, '/admin/templates/')

    def test_get_create_template_view_authenticated(self):
        self.user = TenantUser.objects.get(
            email='AnonymousUser', 
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/templates/create/')
        request.user = self.user
        response = AdminIndexView.as_view()(request)
        
        self.assertEqual(response.status_code, 404)

    def test_update_template_view(self):
        self.user = TenantUser.objects.create_superuser(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123jj@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        self.tenant = Tenant.objects.get(
            name='tenanttest1', 
        )
        request = self.factory.post(f'/admin/templates/{self.tenant.id}/update', data=self.new_tenant_data)
        request.user = self.user
        response = UpdateTemplateView.as_view()(request, pk=self.tenant.id)
        self.assertEqual(response.status_code, 200)


    def test_update_template_unauthenticateduser_view(self):
        self.user = TenantUser.objects.get(
            email='AnonymousUser', 
            )
        self.client.force_login(user=self.user)
        self.tenant = Tenant.objects.get(
            name='tenanttest1', 
        )
        request = self.factory.post(f'/admin/templates/{self.tenant.id}/update', data=self.new_tenant_data)
        request.user = self.user
        response = AdminIndexView.as_view()(request, pk=self.tenant.id)
        self.assertEqual(response.status_code, 404)
    
    def test_delete_template_view(self):
        self.user = TenantUser.objects.create_superuser(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc12ioo3@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        subscription = Subscription.objects.create()
        self.tenant = Tenant.objects.create(name="Test Tenant", type="personal", is_template=False, description="Test tenant for testing purposes", owner=self.user, schema_name='tenantkkop1', subscription=subscription)
        request = self.factory.delete(f'/admin/templates/{self.tenant.id}/delete')
        request.user = self.user
        response = DeleteTemplateView.as_view()(request, pk=self.tenant.id)
        self.assertEqual(response.status_code, 302)

    def test_delete_template_unauthenticateduser_view(self):
        self.user = TenantUser.objects.get(
            email='AnonymousUser', 
            )
        self.client.force_login(user=self.user)
        subscription = Subscription.objects.create()
        self.tenant = Tenant.objects.create(name="Test Tenant", type="personal", is_template=False, description="Test tenant for testing purposes", owner=self.user, schema_name='tenantkkk1', subscription=subscription)
        request = self.factory.delete(f'/admin/templates/{self.tenant.id}/delete')
        request.user = self.user
        response = DeleteTemplateView.as_view()(request, pk=self.tenant.id)
        self.assertEqual(response.status_code, 404)



import os
from multitenancy.apps.forms import TenantForm
from rest_framework import viewsets
from rest_framework.decorators import api_view
from rest_framework.reverse import reverse
from rest_framework.response import Response
from rest_framework import permissions
from django.urls.base import reverse, reverse_lazy
from django.http import HttpResponseRedirect
import sweetify
from tenant_users.tenants.models import InactiveError, ExistsError
from tenant_users.tenants.utils import (get_tenant_model,
                                        get_tenant_domain_model)
from django.conf import settings
from django.shortcuts import redirect, render
from account.mixins import LoginRequiredMixin
from multitenancy.admin.views.baseViews import(
    AdminListView,
    AdminDeleteView,
    AdminCreateView,
    AdminUpdateView,
    AdminTemplateView,
    AdminView,
    AdminDetailView,
    CustomerTemplateView,
    CustomerView
    )
from .filters import TenantFilter
from multitenancy.subscriptions.models import Plan, Subscription
from .models import Tenant
from .serializers import TenantSerializer


class TemplateListView(LoginRequiredMixin ,AdminTemplateView):
    template_name = "multitenancy/apps/template_list.html"

class CreateTemplateView(LoginRequiredMixin, AdminCreateView):
    model = Tenant
    form_class = TenantForm
    success_url = reverse_lazy('template_list')
    template_name = 'multitenancy/apps/create_template.html'

    def post(self, request, is_staff=True, *args, **kwargs):
        form = self.get_form()
        if form.is_valid():
            name = form.cleaned_data["name"]
            tenant_slug = form.cleaned_data["slug"]
            type = form.cleaned_data['type']
            user = request.user

            if not user.is_active:
                raise InactiveError("Inactive user passed to provision tenant")
            tenant_domain = '{}.{}'.format(
                tenant_slug,  settings.TENANT_USERS_DOMAIN)
            DomainModel = get_tenant_domain_model()
            if DomainModel.objects.filter(domain=tenant_domain).exists():
                raise ExistsError('Tenant URL already exists.')
            # time_string = str(int(time.time()))
            # Must be valid postgres schema characters see:
            # https://www.postgresql.org/docs/9.2/static/sql-syntax-lexical.html#SQL-SYNTAX-IDENTIFIERS
            # We generate unique schema names each time so we can keep tenants around without
            # taking up url/schema namespace.
            schema_name = '{}'.format(tenant_slug)
            domain = None
            tenant = None
            try:
                subscription = Subscription.objects.create()

                tenant = Tenant.objects.create(name=name,
                                               slug=tenant_slug,
                                               schema_name=schema_name,
                                               subscription=subscription,
                                               owner=user,
                                               is_template=True,
                                               type=type)
                domain = get_tenant_domain_model().objects.create(domain=tenant_domain,
                                                                  tenant=tenant,
                                                                  is_primary=True)
                tenant.add_user(user, is_superuser=True, is_staff=is_staff)
                tenant.auto_create_schema = False
                tenant.save()
                # Create cursor
                sweetify.success(request, "Successfully Created Tenant!", icon='success', timer=5000)
                # messages.success(request, "Successfully Added Tenant!")
                return reverse_lazy("template_list", urlconf="multitenancy.urls")
            except Exception:
                if domain is not None:
                    domain.delete()
                if tenant is not None:
                    # Flag is set to auto-drop the schema for the tenant
                    tenant.delete(True)
                raise
                return tenant_domain
        # return super().post(request, *args, **kwargs)


class UpdateTemplateView(LoginRequiredMixin ,AdminUpdateView):
    model = Tenant
    form_class = TenantForm
    success_url = reverse_lazy('template_list', urlconf="multitenancy.urls")
    template_name = 'multitenancy/apps/update_template.html'


class DeleteTemplateView(LoginRequiredMixin ,AdminDeleteView):
    model = Tenant
    template_name = "multitenancy/apps/delete_tenant.html"
    success_url = reverse_lazy("template_list", urlconf="multitenancy.urls")

    def delete(self, request, *args, **kwargs):
        tenant_id = self.kwargs['pk']

        tenant = Tenant.objects.filter(id=tenant_id)
        tenant.delete()
        return HttpResponseRedirect(reverse('tenant_list', urlconf="multitenancy.urls"))



class TenantListView(LoginRequiredMixin ,AdminTemplateView):
    template_name = "multitenancy/apps/tenant_list.html"

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        f = TenantFilter(self.request.GET, queryset=Tenant.objects.filter().exclude(is_template=True))
        context['filter'] = f
        return context
    

class CustomerTenantListView(LoginRequiredMixin, CustomerTemplateView):
    template_name = "multitenancy/admin/publicUser/subscriptions.html"
    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)

        subscriptions = TenantFilter(queryset=Tenant.objects.filter(owner=self.request.user))
        context['filter'] = subscriptions
        return context
    

class CreateService(LoginRequiredMixin, CustomerView):
    def get(self,request):
        plans = Plan.objects.all()
        settings.STATICFILES_DIRS = [
            os.path.join(settings.PROJECT_DIR, 'templates/client/static/'),
        ]
        settings.TENANT_CREATION_TEMPLATE ="client/static/publicUser/index.html"
        return render(request,settings.TENANT_CREATION_TEMPLATE, {"plans":plans})

class TenantTemplateViewSet(viewsets.ModelViewSet):
    """
    This viewset automatically provides `list`, `create`, `retrieve`,
    `update` and `destroy` actions.

    Additionally we also provide an extra `highlight` action.
    """
    model = Tenant
    
    serializer_class = TenantSerializer
    permission_classes = [permissions.IsAuthenticated,
                          permissions.IsAdminUser]

    def get_queryset(self):
        query = self.request.GET.get("q") 
        queryset = Tenant.objects.filter(is_template=True).search(query=query)
        return queryset

    def perform_create(self, serializer):
        serializer.save(owner=self.request.user)





from rest_framework.test import APITestCase
from ..serializers import UserSerializer


class UserSerializerTestCase(APITestCase):
    def setUp(self):
        self.valid_data = {
            'first_name': 'John',
            'last_name': 'Doe',
            'username': 'johndoe',
            'email': 'johndoe@example.com',
            'type': 'Customer',
            'signup_confirmation': True,
            'password':'password',
        }
        self.invalid_data = {
            'first_name': 'John',
            'last_name': 'Doe',
            'username': 'johndoe',
            'email': 'invalidemail',
            'type': 'Customer',
            'signup_confirmation': True,
            'password':'password',
        }

    def test_serializer_with_valid_data(self):
        serializer = UserSerializer(data=self.valid_data)# type: ignore        
        self.assertTrue(serializer.is_valid(), msg=serializer.errors)

    def test_serializer_with_invalid_data(self):
        serializer = UserSerializer(data=self.invalid_data)# type: ignore
        self.assertFalse(serializer.is_valid())
        self.assertIn('email', serializer.errors)



# Generated by Django 3.2.12 on 2023-02-08 06:04

from django.db import migrations, models
import phonenumber_field.modelfields


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0007_alter_logo_logo'),
    ]

    operations = [
        migrations.AlterField(
            model_name='generalinfo',
            name='company_name',
            field=models.CharField(blank=True, default='Django Multitenancy', max_length=255),
        ),
        migrations.AlterField(
            model_name='generalinfo',
            name='phone_number',
            field=phonenumber_field.modelfields.PhoneNumberField(blank=True, default='+27212345678', max_length=128, region=None),
        ),
        migrations.AlterField(
            model_name='generalinfo',
            name='website',
            field=models.URLField(blank=True, default='example.com'),
        ),
    ]



from django.dispatch import receiver

from django.db.models.signals import post_save

from django.contrib.auth.models import User

from .models import Profile


@receiver(post_save, sender=User)
def handle_user_save(sender, created, instance, **kwargs):
    if created:
        Profile.objects.get_or_create(user=instance)



from django.shortcuts import render
from django.views.generic import View
from account.mixins import LoginRequiredMixin
from multitenancy.admin.decorators import allowed_users
from .baseViews import (StaffCreateView, StaffListView, TeamListView, StaffUpdateView)
from helpdesk.models import Ticket


class StaffIndexView(View, LoginRequiredMixin):
    @allowed_users(allowed_types=["Staff"])
    def get(self, request, *args, **kwargs):
        active_tickets = Ticket.objects.select_related('queue').exclude(
        status__in=[Ticket.CLOSED_STATUS, Ticket.RESOLVED_STATUS],
        )

        # open & reopened tickets, assigned to current user
        tickets = active_tickets.filter(
            assigned_to=request.user,
        )
        return render(request, 'multitenancy/admin/teamUser/index.html',
                      {
                          'nbar': 'admin',
                          'title': 'Dashboard!',
                        'user_tickets':tickets
                      }
                      )



# Generated by Django 3.2.12 on 2023-02-04 09:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('subscriptions', '0014_subscription_created_date'),
    ]

    operations = [
        migrations.CreateModel(
            name='Coupon',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=15)),
                ('discount', models.DecimalField(decimal_places=2, max_digits=10)),
                ('start_date', models.DateField()),
                ('end_date', models.DateField()),
                ('usage_limit', models.PositiveIntegerField()),
                ('is_active', models.BooleanField(default=True)),
                ('minimum_order_amount', models.DecimalField(decimal_places=2, max_digits=10)),
                ('redeem_by', models.DateField()),
                ('usage_count', models.PositiveIntegerField(default=0)),
            ],
        ),
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateField(auto_now_add=True)),
                ('order_number', models.CharField(max_length=100)),
                ('amount', models.DecimalField(decimal_places=2, max_digits=10)),
                ('status', models.CharField(choices=[('completed', 'Completed'), ('failed', 'Failed')], max_length=10)),
                ('payment_method', models.CharField(max_length=50)),
                ('shipping_address', models.TextField()),
                ('billing_address', models.TextField()),
                ('notes', models.TextField(blank=True)),
                ('coupon', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='order.coupon')),
                ('subscription', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='subscriptions.subscription')),
            ],
        ),
    ]



# Generated by Django 3.2.12 on 2023-01-26 14:55

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0010_auto_20230124_1949'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscription',
            name='end_date',
            field=models.DateField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='product_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='subscriptions', to='subscriptions.producttype'),
        ),
    ]



# Generated by Django 3.2.12 on 2023-01-07 15:42

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0005_alter_plan_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProductFeature',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=250)),
                ('description', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='ProductType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(choices=[('tenant', 'Tenant App'), ('domain', 'Domain'), ('third_party', '3rd party App')], default='tenant', max_length=115)),
                ('features', models.ManyToManyField(to='subscriptions.ProductFeature')),
            ],
        ),
        migrations.RemoveField(
            model_name='subscription',
            name='duration',
        ),
        migrations.AddField(
            model_name='subscription',
            name='cycle',
            field=models.CharField(choices=[('weekly', 'Weekly'), ('monthly', 'Monthly'), ('quartely', 'quarterly'), ('annually', 'annually')], default='monthly', max_length=50),
        ),
        migrations.AddField(
            model_name='subscription',
            name='renewal_date',
            field=models.DateField(null=True),
        ),
        migrations.AddField(
            model_name='subscription',
            name='renewal_status',
            field=models.CharField(choices=[('active', 'Active'), ('cancelled', 'Cancelled'), ('expired', 'Expired')], default='active', max_length=10),
        ),
        migrations.AddField(
            model_name='subscription',
            name='subscription_duration',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='start_date',
            field=models.DateField(auto_now_add=True),
        ),
        migrations.DeleteModel(
            name='UserSubcriptions',
        ),
        migrations.AddField(
            model_name='subscription',
            name='product_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='subscriptions', to='subscriptions.producttype'),
        ),
    ]



# Generated by Django 3.2.12 on 2022-11-12 11:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('users', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Plan',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=250)),
                ('type', models.CharField(max_length=250)),
                ('price', models.DecimalField(decimal_places=2, default=75, max_digits=12, verbose_name='Price')),
            ],
        ),
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.IntegerField()),
                ('duration', models.DateField()),
                ('start_date', models.DateField()),
                ('end_date', models.DateField()),
                ('reference', models.TextField(help_text='Free text field for user references', max_length=100)),
                ('last_updated', models.DateTimeField(auto_now=True, help_text='Keeps track of when a record was last updated')),
                ('reason', models.TextField(help_text='Reason for state change, if applicable.')),
            ],
        ),
        migrations.CreateModel(
            name='UserSubcriptions',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subscriptions', models.ManyToManyField(to='subscriptions.Subscription')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='users.customer')),
            ],
        ),
    ]



# Generated by Django 3.2.12 on 2022-10-30 10:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('profiles', '0004_profile_display_profile'),
    ]

    operations = [
        migrations.AlterField(
            model_name='profile',
            name='skills',
            field=models.ManyToManyField(blank=True, to='profiles.Skills'),
        ),
        migrations.AlterField(
            model_name='profile',
            name='website',
            field=models.URLField(blank=True, max_length=250),
        ),
    ]



from django import forms
from .models import Plan

class PlanForm(forms.ModelForm):

    class Meta:
        model = Plan
        fields = ['name', 'description', 'price']


class ProductFeatureForm(forms.Form):
    name = forms.CharField(label="Name", max_length=250)
    description = forms.CharField(label="Description", max_length=250, widget=forms.Textarea(
        attrs={"class": "form-control"}))
    plan = forms.IntegerField(label="plan", widget=forms.HiddenInput())
    plan_name = forms.CharField(widget=forms.HiddenInput())


# Generated by Django 3.2.12 on 2023-01-01 04:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apps', '0010_auto_20221210_1524'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='tenant',
            options={'verbose_name': 'Service', 'verbose_name_plural': 'Services'},
        ),
        migrations.AlterField(
            model_name='tenant',
            name='type',
            field=models.CharField(choices=[('personal', 'personal'), ('premium', 'premium'), ('business', 'business')], default='personal', max_length=200),
        ),
    ]



from django.db import models
from django.conf import settings
from account.languages import DEFAULT_LANGUAGE
from account.fields import TimeZoneField
from django.utils.translation import gettext_lazy as _
from multitenancy.core.models import BaseSetting
from django_countries.fields import CountryField
from phonenumber_field.modelfields import PhoneNumberField


class SettingsPage:
    def __init__(self, hook_name) -> None:
        pass
    settings_items = []


class Logo(BaseSetting):
    logo = models.ImageField(default="core/logo.png")
    name = 'slug'
    help_text = _('Upload Company Logo')

    class Meta:
        verbose_name = 'Logo'


class GeneralInfo(BaseSetting):
    help_text = 'Update the name, phone number, etc of the company'
    company_name = models.CharField(max_length=255, blank=True, default="Django Multitenancy")
    phone_number = PhoneNumberField(blank=True, default="+27212345678")
    website = models.URLField(blank=True, default="https://example.com")
    email = models.EmailField(null=True, blank=True)

    class Meta:
        verbose_name = 'General Details'


class Address(BaseSetting):
    help_text = 'Update Company Address'
    address_line_1 = models.CharField(max_length=200, null=True, blank=True,)
    address_line_2 = models.CharField(max_length=200, null=True, blank=True)
    city = models.CharField(max_length=200, null=True, blank=True)
    state = models.CharField(max_length=200, null=True, blank=True, verbose_name="State/Province")
    country = CountryField(null=True, blank=True)
    postal_code = models.CharField(max_length=64, null=True, blank=True, verbose_name="Post/Zip-code")

    class Meta:
        verbose_name = 'Address'


class AdminSettings(BaseSetting):
    help_text = 'Update company timezone, currency and other related details.'
    timezone = TimeZoneField(_("timezone"))
    language = models.CharField(_("language"), max_length=10,
                                choices=settings.ACCOUNT_LANGUAGES, default=DEFAULT_LANGUAGE)

    class Meta:
        verbose_name = 'Time zone and currency'

class Currency(BaseSetting):
    code = models.CharField(max_length=3)
    name = models.CharField(max_length=50)
    symbol = models.CharField(max_length=10)
    default = models.BooleanField(default=False)

    def save(self, *args, **kwargs):
        if self.default:
            Currency.objects.filter(default=True).update(default=False)
        super().save(*args, **kwargs)

    def __str__(self):
        return self.code



import hashlib

from django.conf import settings
from django.contrib.auth.hashers import make_password
from django.utils.http import urlencode
from django.contrib.auth import get_user_model
from django.db import connection
from django_tenants.utils import (
    get_public_schema_name,
    get_tenant_domain_model,
    get_tenant_model,
)
from tenant_users.permissions.models import UserTenantPermissions
from tenant_users.tenants.models import ExistsError
from multitenancy.subscriptions.models import Subscription
from multitenancy.apps.models import Tenant

def get_gravatar_url(email, size=50):
    default = "mm"
    size = int(size) * 2  # requested at retina size by default and scaled down at point of use with css
    gravatar_provider_url = getattr(settings, 'MULTITENANCY_GRAVATAR_PROVIDER_URL', '//www.gravatar.com/avatar')

    if (not email) or (gravatar_provider_url is None):
        return None

    gravatar_url = "{gravatar_provider_url}/{hash}?{params}".format(
        gravatar_provider_url=gravatar_provider_url.rstrip('/'),
        hash=hashlib.md5(email.lower().encode('utf-8')).hexdigest(),
        params=urlencode({'s': size, 'd': default})
    )

    return gravatar_url



def create_public_tenant(domain_url, owner_email, password, **owner_extra):
    UserModel = get_user_model()
    TenantModel = get_tenant_model()
    public_schema_name = get_public_schema_name()

    if TenantModel.objects.filter(schema_name=public_schema_name).first():
        raise ExistsError('Public tenant already exists')

    # Create public tenant user. This user doesn't go through object manager
    # create_user function because public tenant does not exist yet
    profile = UserModel.objects.create(
        type="Admin",
        email=owner_email,
        password=make_password(password),
        is_active=True,
        **owner_extra,
    )
    #profile.set_unusable_password()
    profile.save()

    #create a subscription
    subscription = Subscription.objects.create()    

    # Create public tenant
    public_tenant = TenantModel.objects.create(
        schema_name=public_schema_name,
        name='Public Tenant',
        type='public',
        owner=profile,
        subscription=subscription
    )

    

    # Add one or more domains for the tenant
    get_tenant_domain_model().objects.create(
        domain=domain_url,
        tenant=public_tenant,
        is_primary=True,
    )

    # Add system user to public tenant (no permissions)
   # public_tenant.add_user(profile)

    # add user perms
    user_perms = UserTenantPermissions.objects.create(profile_id=profile.pk, is_staff=True, is_superuser=True)


# Generated by Django 3.2.12 on 2022-11-15 06:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GeneralInfo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('company_name', models.CharField(blank=True, max_length=255, null=True)),
                ('phone', models.CharField(blank=True, max_length=255, null=True)),
            ],
            options={
                'verbose_name': 'General Details',
            },
        ),
        migrations.AlterModelOptions(
            name='logo',
            options={'verbose_name': 'Logo'},
        ),
    ]



from django.urls import path
from multitenancy.apps import views

urlpatterns = [
    path("subscriptions/",views.CustomerTenantListView.as_view(), name="subscriptions"),
    path("subscriptions/create/",views.CreateService.as_view(), name="create_service"),
]


from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _

class MultiTenancyAdminConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'multitenancy'
    verbose_name = _("Multitenancy admin")
    app_label = 'multitenancyadmin'



# Generated by Django 3.2.12 on 2023-01-24 17:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0009_alter_plan_slug'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='subscription',
            name='renewal_status',
        ),
        migrations.AlterField(
            model_name='subscription',
            name='product_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='subscriptions', to='subscriptions.producttype'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='status',
            field=models.CharField(choices=[('active', 'Active'), ('inactive', 'Inactive'), ('cancelled', 'Cancelled'), ('expired', 'Expired')], default='inactive', max_length=10),
        ),
    ]



from rest_framework import serializers
from .models import Plan, ProductFeature, Subscription, ProductType

class  ProductFeatureSerializer(serializers.ModelSerializer):
    class Meta:
        model = ProductFeature
        fields = ["id", "name"]

class PlanSerialiser(serializers.ModelSerializer):
    features = ProductFeatureSerializer(many=True)
    class Meta:
        model = Plan
        fields = ["id","name", "description", "price", "price_weekly", "price_quartely", "price_annually", "features"]


class ProductTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = ProductType
        fields = ["id", "name"]

class SubscriptionSerializer(serializers.ModelSerializer):
    product_type = ProductTypeSerializer(many=False)
    class Meta:
        model = Subscription
        fields = ['id', 'cycle', 'subscription_duration', 'start_date', 'end_date', 'created_date', 'renewal_date', 'reference', 'last_updated', 'product_type', 'reason', 'status',]


from django.urls import path, include
from multitenancy.apps import views
from multitenancy.apps import  customer_urls


urlpatterns = [
    path("templates/", views.TemplateListView.as_view(), name="template_list"),
    path("templates/create/", views.CreateTemplateView.as_view(), name="create_template"),
    path("templates/<int:pk>/update/", views.UpdateTemplateView.as_view(), name="update_template"),
    path("templates/<int:pk>/delete/", views.DeleteTemplateView.as_view(), name="delete_tenant"),
    path("tenants/", views.TenantListView.as_view(), name="tenant_list"),
        path('', include(customer_urls)),    
]


from django import forms
from django.contrib.auth.forms import UserCreationForm
from multitenancy.users.models import Customer, Staff, TenantUser

class CustomerForm(forms.ModelForm):
    password = forms.CharField(label='Password', widget=forms.PasswordInput)
    password2 = forms.CharField(label='Confirm Password', widget=forms.PasswordInput)

    class Meta:
        model = TenantUser
        fields = ['first_name', 'last_name', 'username', 'email']
        widgets = {
            'password': forms.PasswordInput(),
            'email': forms.EmailInput(),
        }
    def clean_password2(self):
        # Check that the two password entries match
        password = self.cleaned_data.get("password")
        password2 = self.cleaned_data.get("password2")
        if password and password2 and password != password2:
            raise forms.ValidationError("Passwords don't match")
        return password2

    def save(self, commit=True):
        user = super().save(commit=False)
        user.set_password(self.cleaned_data['password'])
        if commit:
            user.save()
        return user

class CustomerUpdateForm(forms.ModelForm):
    
    class Meta:
        model = TenantUser
        fields = ['first_name', 'last_name', 'username', 'email']
        widgets = {
            'email': forms.EmailInput(),
        }


class StaffForm(forms.ModelForm):
    type = forms.CharField(label='Type', widget=forms.HiddenInput)
    class Meta:
        model = TenantUser
        fields = ['first_name', 'last_name', 'username', 'email', 'password',  'type']
        widgets = {
            'email': forms.EmailInput(),
            'password': forms.PasswordInput(),
        }

    def save(self, commit=True):
        user = super().save(commit=False)
        user.set_password(self.cleaned_data['password'])
        if commit:
            user.save()
        return user

class StaffUpdateForm(forms.ModelForm):

    class Meta:
        model = Staff
        fields = ['first_name', 'last_name', 'username', 'email']
        widgets = {
            'email': forms.EmailInput(),
        }


from rest_framework import serializers

from multitenancy.users.models import TenantUser
from .models import Invoice, Credit, Refund
from multitenancy.subscriptions.serializers import SubscriptionSerializer

class InvoiceSerializer(serializers.ModelSerializer):
    subscription = SubscriptionSerializer(many=False)
    class Meta:
        model = Invoice
        fields = ["id", "subscription", "date_created", "due_date", "invoice_number", "amount", "credit_used", "payment_method", "status", "notes", "pdf"]



class RefundSerializer(serializers.ModelSerializer):
    invoice = InvoiceSerializer(many=False)
    class Meta:
        model = Credit
        fields = "__all__"


class UserDropdownField(serializers.ChoiceField):
    def __init__(self, *args, **kwargs):
        kwargs['choices'] = [(user.id, user) for user in TenantUser.objects.filter().exclude(type="Admin").exclude(type="Staff").exclude(email="AnonymousUser")]
        super().__init__(*args, **kwargs)


class CreditSerializer(serializers.ModelSerializer):
    customer = UserDropdownField()
    class Meta:
        model = Credit
        fields = "__all__"


# Generated by Django 3.2.12 on 2022-11-12 05:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apps', '0004_rename_packages_package'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tenanttype',
            name='name',
            field=models.CharField(choices=[('public', 'public'), ('personal', 'personal'), ('premium', 'premium'), ('business', 'business')], default='personal', max_length=150),
        ),
    ]



# Generated by Django 3.2.12 on 2023-02-03 15:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0013_auto_20230201_2059'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscription',
            name='created_date',
            field=models.DateField(auto_now_add=True, null=True),
        ),
    ]



# Generated by Django 3.2.12 on 2022-11-11 20:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('apps', '0003_auto_20221111_1809'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Packages',
            new_name='Package',
        ),
    ]



# Generated by Django 3.2.12 on 2022-11-15 17:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0003_auto_20221115_1846'),
    ]

    operations = [
        migrations.AddField(
            model_name='address',
            name='slug',
            field=models.SlugField(null=True),
        ),
        migrations.AddField(
            model_name='generalinfo',
            name='slug',
            field=models.SlugField(null=True),
        ),
    ]



from django.urls import path

from multitenancy.admin.views import (
    adminViews,
    authViews
)

#app_name = "multitenancy"

urlpatterns = [
    path(r"", adminViews.AdminIndexView.as_view(), name="admin_dashboard"),
    path("teams/all", adminViews.TeamsIndexView.as_view(), name="teams_index"),    
    path("acounts/login/", authViews.LoginView.as_view(), name="accounts_login"),
    path("page-not-found/", authViews.pageNotFound.as_view(), name="page_not_found"),

]
handler404 = 'multitenancy.admin.views.authViews.pageNotFound'



# Generated by Django 3.2.12 on 2022-12-08 08:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('apps', '0007_tenant_subscription'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='registereddomain',
            name='owner',
        ),
        migrations.RemoveField(
            model_name='registereddomain',
            name='tld',
        ),
        migrations.RemoveField(
            model_name='domain',
            name='dns',
        ),
        migrations.DeleteModel(
            name='DomainTld',
        ),
        migrations.DeleteModel(
            name='RegisteredDomain',
        ),
    ]



# Generated by Django 3.2.12 on 2022-11-11 16:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apps', '0002_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TenantType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150)),
            ],
        ),
        migrations.AddField(
            model_name='tenant',
            name='is_template',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='tenant',
            name='name',
            field=models.CharField(max_length=100),
        ),
        migrations.AlterField(
            model_name='tenant',
            name='type',
            field=models.CharField(max_length=200),
        ),
    ]



from django.http import HttpResponseRedirect
from django.urls.base import reverse
from django.shortcuts import render
import account.forms
import account.views
from django.views.generic import View

class pageNotFound(View):
    def get(self, request, *args, **kwargs):
        return render(request, '404.html',
                      {
                          'nbar': 'admin',
                          'title': 'Dashboard!',

                      }
                      )


class LoginView(account.views.LoginView):

    form_class = account.forms.LoginEmailForm

    def get(self, *args, **kwargs):
        if self.request.user.is_authenticated:
            if self.request.user.type == "Admin":  # type: ignore
                return HttpResponseRedirect(reverse("admin_dashboard"))
            elif self.request.user.type == "Staff":  # type: ignore
                return HttpResponseRedirect(reverse("team_dashboard"))
            else:
                return HttpResponseRedirect(reverse("customer_dashboard"))
            # return redirect(self.get_success_url())
        return super(LoginView, self).get(*args, **kwargs)

    def form_valid(self, form):
        self.login_user(form)
        self.after_login(form)
        if self.request.user.is_authenticated:
            if self.request.user.type == "Admin":  # type: ignore
                return HttpResponseRedirect(reverse("admin_dashboard"))
            elif self.request.user.type == "Staff":  # type: ignore
                return HttpResponseRedirect(reverse("team_dashboard"))
            else:
                return HttpResponseRedirect(reverse("customer_dashboard"))
        # return redirect(self.get_success_url())



# Generated by Django 3.2.12 on 2023-02-13 17:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('billing', '0002_auto_20230209_1635'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='refund',
            options={'ordering': ['-date']},
        ),
    ]



# Generated by Django 3.2.12 on 2022-12-10 13:24

from django.db import migrations, models
import multitenancy.users.models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='tenantuser',
            options={'verbose_name': 'user profile', 'verbose_name_plural': 'user profiles'},
        ),
        migrations.AddField(
            model_name='tenantuser',
            name='avatar',
            field=models.ImageField(blank=True, upload_to=multitenancy.users.models.upload_avatar_to, verbose_name='profile picture'),
        ),
    ]



# Generated by Django 3.2.12 on 2022-10-17 13:13

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('apps', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='tenant',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='tenant',
            name='plan',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to='apps.packages'),
        ),
        migrations.AddField(
            model_name='registereddomain',
            name='owner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='registereddomain',
            name='tld',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='apps.domaintld'),
        ),
        migrations.AlterUniqueTogether(
            name='domaintld',
            unique_together={('name', 'price')},
        ),
        migrations.AddField(
            model_name='domain',
            name='dns',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, to='apps.registereddomain'),
        ),
        migrations.AddField(
            model_name='domain',
            name='tenant',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='domains', to='apps.tenant'),
        ),
        migrations.AlterUniqueTogether(
            name='tenant',
            unique_together={('id', 'name')},
        ),
    ]



from django.urls import path
from . import views

urlpatterns = [
    path("settings/", views.SettingsIndexView.as_view(), name="settings_index"),
    path("settings/general/", views.SettingsView.as_view(), name="generalsettings_index"),
    path("settings/general/logo/", views.UpdateLogoView.as_view(), name="update_logo"),
    path("settings/general/info/", views.GeneralInfoView.as_view(), name="update_info"),
    path("settings/general/address/", views.AddressView.as_view(), name="update_address"),
    path("settings/general/adminsettings/", views.AdminSettingsView.as_view(), name="update_adminsettings"),
]


# Generated by Django 3.2.12 on 2023-01-26 14:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0002_auto_20221210_1524'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tenantuser',
            name='type',
            field=models.CharField(choices=[('Admin', 'Admin'), ('Staff', 'Staff'), ('Customer', 'Customer')], default='Customer', max_length=255, verbose_name='Type'),
        ),
    ]



# Generated by Django 3.2.12 on 2022-10-07 07:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('profiles', '0002_profile_job_title'),
    ]

    operations = [
        migrations.CreateModel(
            name='Skills',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150)),
            ],
        ),
        migrations.RenameField(
            model_name='profile',
            old_name='affiliation',
            new_name='education',
        ),
        migrations.AddField(
            model_name='profile',
            name='skills',
            field=models.ManyToManyField(to='profiles.Skills'),
        ),
    ]



# Generated by Django 3.2.12 on 2022-11-13 09:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0003_auto_20221113_0746'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='plan',
            name='type',
        ),
        migrations.AddField(
            model_name='plan',
            name='description',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from multitenancy.admin.models import Admin, register_admin
from .models import Tenant,Domain


@register_admin
class TenantAdmin(Admin):
    model = Tenant
    menu_label = 'Templates'
    menu_order = 200
    menu_icon = 'fas fa-copy'
    list_display = ['id', 'name', 'type', 'is_template', 'plan', 'description', 'subscription', 'trail_duration', 'on_trial', 'created', 'modified']    

@register_admin
class DomainAdmin(Admin):
    model = Domain
    menu_label = 'My Model'
    menu_order = 200
    menu_icon = 'site'
    list_display = ['id', 'name', 'type', 'is_template', 'plan', 'description', 'subscription', 'trail_duration', 'on_trial', 'created', 'modified']    




# Generated by Django 3.2.12 on 2023-01-24 17:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apps', '0012_auto_20230107_1742'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tenant',
            name='trail_duration',
            field=models.IntegerField(default=0),
        ),
    ]



import re

from django import forms

from .models import Profile
from django_select2 import forms as s2forms


class SkillsWidget(s2forms.ModelSelect2MultipleWidget):
    search_fields = [
        "name__icontains",
    ]


class ProfileForm(forms.ModelForm):

    class Meta:
        model = Profile
        fields = [
            "name",
            "job_title",
            
            "bio",
            "skills",
            "display_profile",
            "education",
            "location",
            "website",
            "twitter_username",
        ]
        widgets = {
            'skills': SkillsWidget,

        }

    def clean_twitter_username(self):
        value = self.cleaned_data["twitter_username"]
        value = value.strip()
        if not value:
            return value
        if value.startswith("@"):
            value = value[1:]
        m = re.match(r"^[a-zA-Z0-9_]{1,20}$", value)
        if not m:
            raise forms.ValidationError("invalid Twitter username")
        return value



from django.conf.urls import url
from .views import ProfileDetailView, ProfileEditView, ProfileListView


urlpatterns = [
    url(r"^profile/edit/", ProfileEditView.as_view(), name="profiles_edit"),
    url(r"^u/$", ProfileListView.as_view(), name="profiles_list"),
    url(r"^u/(?P<username>[\w\._-]+)/$", ProfileDetailView.as_view(), name="profiles_detail"),
]



from decimal import Decimal
from django.conf import settings
from django.db import models
from multitenancy.subscriptions.models import Subscription

from django.conf import settings
from django.db.models import Q

class InvoiceQuerySet(models.QuerySet):
    def search(self, query=None):
        if query is None or query =="":
                return self.all()
        lookups = Q(subscription__product_type__name__icontains=query) | Q(id__exact=query) | Q(invoice_number__exact=query)
        return self.filter(lookups)
    
    def filter_by_id(self, id=None):
            if id is None or id =="":
                return self.all()
            return self.filter(id__exact=id)
    
    def get_due_date(self, date=None):
        if date is None or date =="":
                return self.all()
        return self.filter(due_date__exact=date)
    
    def filter_date_range(self, start_date=None, end_date=None):
        if start_date is None and end_date is None:
              return self.all()
        return self.filter(date_created__range=(start_date, end_date))
        
    def get_status(self, status):
        if status is None or status =="":
                return self.all()
        return self.filter(status__exact=status)
    

class InvoiceManager(models.Manager):
    def get_queryset(self):
        return InvoiceQuerySet(self.model, using=self._db)

    def search(self, query=None):
        return self.get_queryset().search(query=query)
    
    def filter_by_id(self, id=None):
        return self.get_queryset().filter_by_id(id=id)
    
    def get_due_date(self, date=None):
         return self.get_queryset().get_due_date(date=date)
    
    def filter_date_range(self, start_date, end_date):
         return self.get_queryset().filter_date_range(start_date=start_date, end_date=end_date)
    
    def get_status(self, status):
        return self.get_queryset().get_status(status=status)

class Invoice(models.Model):
    subscription = models.ForeignKey(Subscription, on_delete=models.CASCADE)
    date_created = models.DateField(auto_now_add=True)
    due_date = models.DateField()
    invoice_number = models.CharField(max_length=100)
    amount = models.DecimalField(max_digits=10, decimal_places=2)
    credit_used = models.DecimalField(max_digits=10, decimal_places=2, default=Decimal(0))
    payment_method = models.CharField(max_length=50)
    status = models.CharField(max_length=10, choices=[("paid", "Paid"), ("unpaid", "Unpaid"),("pending", "Pending"),("cancelled", "Cancelled"),("refunded", "Refunded")])
    notes = models.TextField(blank=True)
    pdf = models.FileField(upload_to='invoices/')
    objects = InvoiceManager()

    class Meta:
        ordering = ['-date_created']

    def __str__(self):
        return self.invoice_number
    
    def create_pdf(self):
        pass


class RefundQuerySet(models.QuerySet):
    def get_invoice(self, invoice_id=None):
        if invoice_id is None or invoice_id == "":
            return self.all()
        return self.filter(invoice_id__exact=invoice_id)
    
class RefundManager(models.Manager):
    def get_queryset(self):
        return RefundQuerySet(self.model, using=self._db)
    
    def get_invoice(self, invoice_id=None):
        return self.get_queryset().get_invoice(invoice_id=invoice_id)

class Refund(models.Model):
    invoice = models.ForeignKey(Invoice, on_delete=models.CASCADE)
    amount = models.DecimalField(max_digits=10, decimal_places=2)
    date = models.DateTimeField(auto_now_add=True)
    reason = models.TextField()
    objects = RefundManager()

    class Meta:
        ordering = ["-date"]

    def __str__(self) -> str:
        return f"Invoice -{self.invoice.pk}, Amount: {self.amount}"

class CreditQuesrySet(models.QuerySet):
    def get_customer_credit(self, customer_id=None):
        if customer_id is None or customer_id == "":
            return self.all()
        return self.filter(customer_id__exact=customer_id)
    
class CreditManager(models.Manager):
    def get_queryset(self):
        return CreditQuesrySet(self.model, using=self._db)
    
    def get_customer_credit(self, customer_id=None):
        return self.get_queryset().get_customer_credit(customer_id=customer_id)

class Credit(models.Model):
    customer = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    amount = models.DecimalField(max_digits=10, decimal_places=2)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    objects = CreditManager()

    def __str__(self) -> str:
        return f"{self.customer}, {self.amount}"


class PaymentGateWay(models.Model):
    name = models.CharField(max_length=250)
    description = models.TextField()
    status = models.CharField(max_length=10, choices=[("active", "Active"), ("inactive", "Inactive")])
    default = models.BooleanField(default=False)


class Payment(models.Model):
    subscriber = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    amount = models.DecimalField(max_digits=10, decimal_places=2)
    gateway = models.ForeignKey(PaymentGateWay, on_delete=models.PROTECT)
    status = models.CharField(max_length=10, choices=[("success", "Success"), ("failed", "Failed"), ("pending", "Pending")])

class PaymentHandler:
    def __init__(self, payment_gateway):
        self.payment_gateway = payment_gateway

    def process_payment(self, payment, amount):
        """
        Process a payment through the payment gateway.
        :param payment: The Payment object representing the payment.
        :param amount: The amount of the payment.
        :return: The result of the payment processing.
        """
        # Code to communicate with the payment gateway API and process the payment goes here
        result = {'success': True, 'message': 'Payment processed successfully.'}
        return result

    def refund_payment(self, payment, amount):
        """
        Refund a payment through the payment gateway.
        :param payment: The Payment object representing the payment.
        :param amount: The amount of the refund.
        :return: The result of the refund processing.
        """
        # Code to communicate with the payment gateway API and process the refund goes here
        result = {'success': True, 'message': 'Refund processed successfully.'}
        return result





"""
Django settings for {{ project_name }} project.
Generated by 'django-admin startproject' using Django {{ django_version }}.
For more information on this file, see
https://docs.djangoproject.com/en/{{ docs_version }}/topics/settings/
For the full list of settings and their values, see
https://docs.djangoproject.com/en/{{ docs_version }}/ref/settings/
"""

import os
from pathlib import Path
from django.utils.translation import gettext_lazy as _
# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent
import sys
sys.setrecursionlimit(10000)
PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))



import environ
 
env = environ.Env()
TENANT_APPS_DIR = os.path.join(PROJECT_DIR, os.pardir)
sys.path.insert(0, TENANT_APPS_DIR)

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.2/howto/deployment/checklist/


ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

ALLOWED_HOSTS = ['*']

SITE_ID = 1

# Application definition

HAS_MULTI_TYPE_TENANTS = True
MULTI_TYPE_DATABASE_FIELD = 'type'  # or whatever the name you call the database field

TENANT_TYPES = {
    "public": {  # this is the name of the public schema from get_public_schema_name
        "APPS": [
            # mandatory
            'django_tenants',
            'tenant_users.permissions', # Defined in both shared apps and tenant apps
            'tenant_users.tenants.apps.TenantsConfig', # defined only in shared apps
            
            # multitenancy
            'multitenancy.users',
            'multitenancy.core',
            'multitenancy.apps',
            'multitenancy.admin',
            'multitenancy.group.apps.GroupConfig',
            'multitenancy.profiles',
            'multitenancy.subscriptions',
            'multitenancy.billing',
            'multitenancy.order',
            'multitenancy.address',
            'multitenancy.settings',

            # 3rd party apps
            'rest_framework',
            'reversion',
            'django_countries',
            "phonenumber_field",
            'django_crontab',
            'django_redis',
            'django_select2',
            "bootstrap4",
            'formtools',
            'sweetify',
            'widget_tweaks',
            'django_extensions',
            'guardian',
            'groups_manager',
            'account',
            'pinax.teams',
            'pinax.invitations',
            'django.contrib.humanize',  # Required for elapsed time formatting
            'markdown_deux',  # Required for Knowledgebase item formatting
            'bootstrap4form', # Required for nicer formatting of forms with the default templates
            'helpdesk', # Required for helpdesk
            # Django
            'django.contrib.sites',
            'django.contrib.admin',
            'django.contrib.sessions',
            'django.contrib.messages',
            'django.contrib.sitemaps',
            'django.contrib.auth',
            'django.contrib.staticfiles',
            'django.contrib.contenttypes',
                  # shared apps here
                  ],
        "URLCONF": "{{ project_name }}.public_urls", # url for the public type here
    },
    "basic": {
        "APPS": [
            'tenant_users.permissions',

            'rest_framework',

            'storages',
            # Dango
            'django.contrib.sites',
            'django.contrib.auth',
            'django.contrib.messages',
            'django.contrib.sitemaps',
            'django.contrib.staticfiles',
            'django.contrib.contenttypes',
                 # type1 apps here
                 ],
        "URLCONF": "{{project_name}}.urls_basic",
    },
    "premium": {
        "APPS": [
                'rest_framework',
                'tenant_users.permissions',


                

                # Dango
                'django.contrib.sites',
                'django.contrib.auth',
                'django.contrib.messages',
                'django.contrib.sitemaps',
                'django.contrib.staticfiles',
                'django.contrib.contenttypes',
                 # type1 apps here
                 ],
        "URLCONF": "{{project_name}}.urls_premium",
    },
    "business": {
        "APPS": [
                'rest_framework',
                'tenant_users.permissions',




                # Dango
                'django.contrib.sites',
                'django.contrib.auth',
                'django.contrib.messages',
                'django.contrib.sitemaps',
                'django.contrib.staticfiles',
                'django.contrib.contenttypes',
                 # type1 apps here
                 ],
        "URLCONF": "{{project_name}}.urls_business",
    }
}


INSTALLED_APPS = []
for schema in TENANT_TYPES:
    INSTALLED_APPS += [app for app in TENANT_TYPES[schema]["APPS"] if app not in INSTALLED_APPS]

MIDDLEWARE = [
    'django_tenants.middleware.main.TenantMainMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    "corsheaders.middleware.CorsMiddleware",
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    "multitenancy.settings.middleware.TeamMiddleware",

]

MIDDLEWARE_CLASSES = [
#    "account.middleware.ExpiredPasswordMiddleware",
    "account.middleware.ExpiredPasswordMiddleware",
    "account.middleware.LocaleMiddleware",
    "account.middleware.TimezoneMiddleware",

]

ACCOUNT_EMAIL_UNIQUE = False
ACCOUNT_EMAIL_CONFIRMATION_REQUIRED = False


LOGIN_URL = "accounts_login"
LOGIN_REDIRECT_URL="accounts_login"
ACCOUNT_LOGOUT_REDIRECT_URL = "accounts_login"



DATABASE_ROUTERS = (
    'django_tenants.routers.TenantSyncRouter',
)

TENANT_MODEL = "apps.Tenant"  # app.Model

TENANT_DOMAIN_MODEL = "apps.Domain"  # app.Model

AUTH_USER_MODEL = 'users.TenantUser'

AUTHENTICATION_BACKENDS = (
    'tenant_users.permissions.backend.UserBackend',
    'account.auth_backends.EmailAuthenticationBackend',
    'guardian.backends.ObjectPermissionBackend',

)
ROOT_URLCONF = '{{project_name}}.urls'



TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS':[os.path.join(BASE_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                "account.context_processors.account",
            ],
            
        },
    },
]


GROUPS_MANAGER = {
    'AUTH_MODELS_SYNC': True,
}

ACCOUNT_LOGIN_REDIRECT_URL = 'account_login'

ACCOUNT_EMAIL_REQUIRED=True

DATA_UPLOAD_MAX_NUMBER_FIELDS = 10240 
TENANT_LIMIT_SET_CALLS = True

ACCOUNT_AUTHENTICATION_METHOD = "email"

LANGUAGE_CODE = 'en-us'

LANGUAGES = [
    ('en-us', _('English'))
]

WSGI_APPLICATION = '{{project_name}}.wsgi.application'

GROUPS_MANAGER = {
'AUTH_MODELS_SYNC': True,
}

CACHES = {
    # … default cache config and others
    "default": {
                "BACKEND": "django_redis.cache.RedisCache",
                "LOCATION": "redis://127.0.0.1:6379/1",
                "OPTIONS": {
                    "CLIENT_CLASS": "django_redis.client.DefaultClient",
                }
            },
    "select2": {
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/2",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    }
}

# Tell select2 which cache configuration to use:
SELECT2_CACHE_BACKEND = "select2"
SELECT2_JS = "https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.12/js/select2.min.js"
SELECT2_CSS = "https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.12/css/select2.min.css"
SELECT2_I18N_PATH = "https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.12/js/i18n"
# Database
 # https://docs.djangoproject.com/en/3.2/ref/settings/#databases




# Password validation
# https://docs.djangoproject.com/en/3.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]



# Internationalization
# https://docs.djangoproject.com/en/3.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.2/howto/static-files/

MEDIA_ROOT=os.path.join(PROJECT_DIR,'media/')



MEDIA_URL= '/media/'

STATIC_URL="/static/"
STATIC_ROOT=os.path.join(PROJECT_DIR,"static")

STATICFILES_FINDERS = [
    "django_tenants.staticfiles.finders.TenantFileSystemFinder",  # Must be first
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    "compressor.finders.CompressorFinder",
]


MULTITENANT_STATICFILES_DIRS = [
    os.path.join( PROJECT_DIR, "tenants/%s/static" ),
]

STATICFILES_STORAGE = "django_tenants.staticfiles.storage.TenantStaticFilesStorage"

MULTITENANT_RELATIVE_STATIC_ROOT = ""  # (default: create sub-directory for each tenant)

MULTITENANT_RELATIVE_MEDIA_ROOT = ""


DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'



TAGGIT_CASE_INSENSITIVE = True
HELPDESK_PUBLIC_TICKET_DUE_DATE = False
HELPDESK_VIEW_A_TICKET_PUBLIC = True



from .base import *

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True
# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "{{ secret_key }}"

# SECURITY WARNING: define the correct hosts in production!
ALLOWED_HOSTS = ["*"]

EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"

TENANT_USERS_DOMAIN = "{{domain}}"



DATABASES = {
    'default': {
        'ENGINE': 'django_tenants.postgresql_backend',
        'NAME': '{{database}}',
        'USER': 'postgres',
        'PASSWORD': '{{password}}',
        'HOST': 'localhost',
        'PORT': '{{port}}'
    }
}


try:
    from .local import *
except ImportError:
    pass


import re
from django.utils.deprecation import MiddlewareMixin
from django.http import Http404

from account.utils import handle_redirect_to_login

from pinax.teams.models import Team, Membership
from pinax.teams.conf import settings

def check_team_allowed(request):
    allowed = [
        r"^/teams/[\w-]+/account/login/$",
        r"^/teams/[\w-]+/account/signup/",
        r"^/teams/[\w-]+/ajax/username-validation/",
        r"^/teams/[\w-]+/account/password/reset/",
    ]
    for allow_re in allowed:
        if re.search(allow_re, request.path):
            return None
    return handle_redirect_to_login(request, redirect_field_name="next")


class TeamMiddleware(MiddlewareMixin):

    def process_request(self, request):
        team_slug = request.environ.get("pinax.team")
        if team_slug is not None:
            try:
                team = Team.objects.get(slug=team_slug)
            except Team.DoesNotExist:
                if request.user.is_authenticated:
                    request.user.teams = None
                    raise Http404()
                else:
                    return check_team_allowed(request)
            else:
                request.team = team
        else:
            request.team = None
        if request.user.is_authenticated and settings.PINAX_TEAMS_PROFILE_MODEL:
            if re.search(r"^/teams/[\w-]+/account/signup/", request.path):
                return None
            profiles = settings.PINAX_TEAMS_PROFILE_MODEL.objects.filter(
                user=request.user
            )
            request.user.teams = profiles.filter(
                team__isnull=False,
                team__memberships__state__in=[
                    Membership.STATE_ACCEPTED,
                    Membership.STATE_AUTO_JOINED
                ]
            ).distinct()
            try:
                request.profile = profiles.get(team=request.team)
            except settings.PINAX_TEAMS_PROFILE_MODEL.DoesNotExist:
                raise Http404()
        else:
            if team_slug is not None:
                return check_team_allowed(request)


from multitenancy.subscriptions.models import Plan
from multitenancy.subscriptions.views import (
    CreatePlanView, 
    PlanDetailView,
    PlanListView,
)
import unittest
from django.test import RequestFactory, TestCase, Client

from multitenancy.users.models import TenantUser

class PlanViewsTestCase(unittest.TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.client = Client()
        
    def test_get_create_plan_view(self):
        self.user = TenantUser.objects.create_superuser(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='testaadminabc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/settings/plans/create/')
        request.user = self.user
        response = CreatePlanView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        
                                              
    def test_post_create_plan_view(self):
        self.user = TenantUser.objects.create_superuser(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='testadminabc123@email.com', 
            type='Admin',
            is_active=True
            )
        self.client.force_login(user=self.user)
        self.data= {
            "name":"basic",
            "description":"basic plan",
            "price":78,
        }
        request = self.factory.post('/admin/plans/create/', data=self.data)
        request.user = self.user
        response = CreatePlanView.as_view()(request)
        self.assertEqual(response.status_code, 200)

    
    def test_plan_detail_view(self):
        self.admin = TenantUser.objects.create_superuser(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='test2adminabc123@email.com', 
            type='Admin',
            is_active=True
            )
        self.client.force_login(user=self.admin)
        self.plan = Plan.objects.create(
            name='enterprise',
        )
        self.request = self.factory.get(f'/settings/plans/{self.plan}/')
        self.request.user = self.admin
        view = PlanDetailView.as_view()
        response = view(self.request, pk=self.plan.pk)
        self.assertEqual(response.status_code, 200)
        



import unittest
from django.test import RequestFactory, TestCase, Client
from django.urls import reverse, reverse_lazy
from tenant_users.permissions.models import UserTenantPermissions
from multitenancy.admin.views.adminViews import (
    AdminIndexView, 
    )
from multitenancy.settings.views import SettingsIndexView
from multitenancy.apps.views import TenantListView
from multitenancy.subscriptions.views import (
    CreatePlanView, 
    PlanDetailView,
    PlanListView,
)
from multitenancy.users.views import(
    CreateCustomerView,
    DeleteCustomerView,
    CustomerListView,
    UpdateCustomerView,
)
from multitenancy.subscriptions.models import Plan
from multitenancy.users.models import Admin, Customer, TenantUser
from  multitenancy.admin.decorators import allowed_users
from multitenancy.users.forms import CustomerForm, CustomerUpdateForm
from multitenancy.subscriptions.forms import PlanForm
from multitenancy.admin import urls



class AdminViewsTestCase(unittest.TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.client = Client()
        self.customer_data = {
            'first_name': 'customer',
            'last_name': 'test',
            'username': 'tcustomer',
            'password': 'password1234',
            'email': 'customer@example.com',
        }
        self.new_customer_data = {
            'first_name': 'customer1',
            'username': 'testcustomer',
        }
        self.customer_obj = Customer.objects.first()


    
    def test_admin_index_view(self):
        self.user = TenantUser.objects.create(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/')
        request.user = self.user
        response = AdminIndexView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(self.client.session.get('_auth_user_id'))
        self.assertIn('_auth_user_id', self.client.session)

    def test_admin_index_view_authenticated(self):
        self.user = TenantUser.objects.get(
            
            email='AnonymousUser', 
            
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/')
        request.user = self.user
        response = AdminIndexView.as_view()(request)
        
        self.assertEqual(response.status_code, 404)
    

    

     
    

    def test_subscriptionlist_index_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/tenants/')
        request.user = self.user
        response = TenantListView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(self.client.session.get('_auth_user_id'))
        self.assertIn('_auth_user_id', self.client.session)

    def test_subscriptionlist_index_view_authenticated(self):
        self.user = TenantUser.objects.get(
            
            email='AnonymousUser', 
            
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/tenants/')
        request.user = self.user
        response = TenantListView.as_view()(request)
        
        self.assertEqual(response.status_code, 404)



    def test_customerlist_index_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/customers/')
        request.user = self.user
        response = CustomerListView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(self.client.session.get('_auth_user_id'))
        self.assertIn('_auth_user_id', self.client.session)

    def test_customerlist_index_view_authenticated(self):
        self.user = TenantUser.objects.get(
            
            email='AnonymousUser', 
            
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/customers/')
        request.user = self.user
        response = CustomerListView.as_view()(request)
        
        self.assertEqual(response.status_code, 404)


    def test_settings_index_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/settings/')
        request.user = self.user
        response = SettingsIndexView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(self.client.session.get('_auth_user_id'))
        self.assertIn('_auth_user_id', self.client.session)

    def test_settings_index_view_authenticated(self):
        self.user = TenantUser.objects.get(
            
            email='AnonymousUser', 
            
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/settings/')
        request.user = self.user
        response = SettingsIndexView.as_view()(request)
        
        self.assertEqual(response.status_code, 404)


    def test_planlist_index_view(self):
        self.user = TenantUser.objects.get(
            username='admin', 
            password="password", 
            first_name='abc123', 
            last_name='khamban', 
            email='abc123@email.com', 
            type='Admin',
            is_active=True
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/settings/plans/')
        request.user = self.user
        response = PlanListView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(self.client.session.get('_auth_user_id'))
        self.assertIn('_auth_user_id', self.client.session)

    def test_planlist_view_authenticated(self):
        self.user = TenantUser.objects.get(
            
            email='AnonymousUser', 
            
            )
        
        self.client.force_login(user=self.user)
        request = self.factory.get('/admin/settings/plans/')
        request.user = self.user
        response = PlanListView.as_view()(request)
        
        self.assertEqual(response.status_code, 404)


    


# Generated by Django 3.2.12 on 2023-01-31 19:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0011_auto_20230126_1655'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscription',
            name='subscription_duration',
            field=models.IntegerField(default=0),
        ),
    ]



import json
import uuid
from django.views.generic import View
from django.http.response import JsonResponse
from rest_framework.decorators import api_view
from rest_framework.response import Response
from django.urls.base import reverse, reverse_lazy
from rest_framework import viewsets
from rest_framework.reverse import reverse
from rest_framework import permissions
from django.http import HttpResponseRedirect
from django.shortcuts import redirect, render
from django.conf import settings
from tenant_users.tenants.utils import (get_tenant_model,
                                        get_tenant_domain_model)
from tenant_users.tenants.models import InactiveError, ExistsError
import sweetify
from account.mixins import LoginRequiredMixin
from multitenancy.subscriptions.filters import PlanFilter
from multitenancy.subscriptions.forms import PlanForm, ProductFeatureForm
from multitenancy.subscriptions.models import Plan, ProductType, Subscription, ProductFeature
from multitenancy.apps.models import Tenant
from multitenancy.subscriptions.serializers import PlanSerialiser, SubscriptionSerializer
from multitenancy.admin.views.baseViews import CustomerView
from multitenancy.admin.views.baseViews import(
    AdminListView,
    AdminDeleteView,
    AdminCreateView,
    AdminUpdateView,
    AdminTemplateView,
    AdminView,
    AdminDetailView
    )

class PlanListView(LoginRequiredMixin ,AdminListView):
    
    model = Plan
    template_name = 'multitenancy/subscriptions/plan_list.html'

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        f = PlanFilter(self.request.GET, queryset=Plan.objects.all())
        context['filter'] = f
        return context


class CreatePlanView(LoginRequiredMixin ,AdminCreateView):
    model = Plan
    form_class = PlanForm
    success_url = reverse_lazy('plan_list', urlconf="multitenancy.urls")
    template_name = 'multitenancy/subscriptions/create_plan.html'


class PlanDetailView(LoginRequiredMixin ,AdminDetailView):
    template_name = "multitenancy/subscriptions/plan_detail.html"
    model = Plan
    # context_object_name = "plan"
    #slug_field = "slug"
    # def get_context_data(self, slug,*args, **kwargs):
    #     context = super().get_context_data(*args, **kwargs)
    #     plan = Plan.objects.get(slug=slug)
    #     print(plan.features)
    #     context['plan'] = plan
    #     context['form'] =  ProductFeatureForm()
    #     return context

class FeatureCreateView(LoginRequiredMixin, AdminView):

    
    def post(self, request, *args: str, **kwargs):
        if request.method != "POST":
            return HttpResponseRedirect("Method Not Allowed")
        else:
            form = ProductFeatureForm(request.POST)
            print(request.content_params)
            if form.is_valid():
                name = form.cleaned_data["name"]
                description = form.cleaned_data["description"]
                
                plan_name = form.cleaned_data["plan_name"]
                plan = form.cleaned_data["plan"]
                feature = None
                
                try:
                    feature = ProductFeature.objects.create(name=name, description=description)
                    feature.save()
                    print(feature.pk)
                    feature_plan = Plan.objects.get(id=plan)
                    feature_plan.features.add(feature)
                    print(feature_plan.pk)
                    feature_plan.add_feature(feature.pk)
                    
                    sweetify.success(request, "Successfully Added Feature!", icon='success', timer=5000)
                    return HttpResponseRedirect(f"/admin/settings/plans/{plan_name}/")
                except:                               
                    sweetify.error(request, "Failed to Add Feature!")
                    return HttpResponseRedirect(f"/admin/settings/plans/{plan_name}/")
            else:
                form=ProductFeatureForm(request.POST)
                return render(request, 'multitenancy/subscriptions/plan_detail.html', {"feature_form":form})



class UpdatePlanView(LoginRequiredMixin, AdminUpdateView):
    model = Plan
    form_class = PlanForm
    success_url = reverse_lazy('plan_list', urlconf="multitenancy.urls")
    template_name = 'multitenancy/subscriptions/update_plan.html'


class DeletePlanView(LoginRequiredMixin, AdminDeleteView):
    model = Plan
    template_name = "multitenancy/subscriptions/delete_plan.html"
    success_url = reverse_lazy("plan_list", urlconf="multitenancy.urls")


class UserSubscriptionsListView(LoginRequiredMixin, AdminListView):
    model = Subscription
    template_name = 'multitenancy/subscriptions/usersubscriptions_list.html'


class PlanViewSet(viewsets.ModelViewSet):
    """
    This viewset automatically provides `list`, `create`, `retrieve`,
    `update` and `destroy` actions.

    Additionally we also provide an extra `highlight` action.
    """
    queryset = Plan.objects.all()
    serializer_class = PlanSerialiser
    # permission_classes = [permissions.IsAuthenticated,
    #                       permissions.IsAdminUser]

    def perform_create(self, serializer):
        serializer.save(owner=self.request.user)

class ProductTypeListView(LoginRequiredMixin, AdminListView):
    model = ProductType

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        
        
        context['object_list'] = ProductType.objects.all()
        return context

    template_name = 'multitenancy/subscriptions/product_type_list.html'


class SubscriptionsViewSet(viewsets.ModelViewSet):
    """
    This viewset automatically provides `list`, `create`, `retrieve`,
    `update` and `destroy` actions.

    Additionally we also provide an extra `highlight` action.
    """
    queryset = Subscription.objects.all()
    serializer_class = SubscriptionSerializer
    permission_classes = [permissions.IsAuthenticated,
                          permissions.IsAdminUser]

    def perform_create(self, serializer):
        serializer.save(owner=self.request.user)


@api_view(['GET'])
def api_root(request, format=None):
    return Response({
        'plans': reverse('plan-list', request=request, format=format),
        'subscriptions': reverse('subscription-list', request=request, format=format)
    })



class PlanView(CustomerView):

    def post(self, request):
        data = json.loads(request.body)
        duration = data.get('subscription_duration')
        plan = data.get('plan')
        plan_type = Plan.objects.get(id=plan)
        type = plan_type.name
        user = request.user
        randon_id = str(uuid.uuid1())
        
        
        tenant_slug = f"{user.username}{randon_id}"
        print(tenant_slug)
        if not user.is_active:
            raise InactiveError("Inactive user passed to provision tenant")
        tenant_domain = '{}.{}'.format(
            tenant_slug,settings.TENANT_USERS_DOMAIN)
        DomainModel = get_tenant_domain_model()
        if DomainModel.objects.filter(domain=tenant_domain).exists():
            raise ExistsError('Tenant URL already exists.')
        
        schema_name = '{}'.format(tenant_slug)
        domain = None
        tenant = None
        try:
            subscription = Subscription.objects.create()
            subscription.start_subscription("monthly")

            tenant = Tenant.objects.create(name=tenant_domain,
                                            slug=tenant_slug,
                                            schema_name=schema_name,
                                            subscription=subscription,
                                            owner=user,
                                            is_template=True,
                                            type=type)
            domain = get_tenant_domain_model().objects.create(domain=tenant_domain,
                                                                tenant=tenant,
                                                                is_primary=True)
            tenant.add_user(user, is_superuser=True, is_staff=True)
            tenant.auto_create_schema = False
            tenant.save()
            # Create cursor
            sweetify.success(request, "Successfully Created Tenant!", icon='success', timer=5000)
            
            # Do something with the selected plan and duration
            # e.g. create a new subscription in the database
            return JsonResponse({"plan": plan, "subscription_duration": duration})
        except Exception:
            if domain is not None:
                domain.delete()
            if tenant is not None:
                # Flag is set to auto-drop the schema for the tenant
                tenant.delete(True)
            raise
            return tenant_domain        
        



import django_filters
from multitenancy.apps.models import Tenant

from multitenancy.users.models import Customer



class CustomerFilter(django_filters.FilterSet):
    class Meta:
        model = Customer
        fields = ['id']



# Generated by Django 3.2.12 on 2023-01-08 13:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0007_auto_20230108_1622'),
    ]

    operations = [
        migrations.AddField(
            model_name='plan',
            name='slug',
            field=models.SlugField(null=True),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='subscription_duration',
            field=models.IntegerField(default=30),
        ),
    ]



import unittest
from multitenancy.users.models import Admin, Customer, Staff, TenantUser
from tenant_users.permissions.models import UserTenantPermissions


class StaffTestCase(unittest.TestCase):

        

    def test_staff_exists(self):
        staff_a = Staff.objects.create(
            username='staff1', first_name='abc', last_name='kham', email='abcstaff@email.com',
        )
        
        staff_a_pass = 'somepasswaord'
        staff_a.set_password(staff_a_pass)
        self.staff_a_pass = staff_a_pass
        staff_a.save()
        staff_a.add_user_perms()
        self.staff_a = staff_a
        staff_count = Staff.objects.all().count()
        self.assertEqual(staff_count, 1)
        self.assertNotEqual(staff_count, 0)

    def test_staff_password(self):
        staff_a = Staff.objects.get(
            username='staff1', first_name='abc', last_name='kham', email='abcstaff@email.com',
        )
        
        staff_a_pass = 'somepasswaord'
        staff_a.set_password(staff_a_pass)
        self.staff_a_pass = staff_a_pass
        staff_a.save()
        
        self.staff_a = staff_a
        staff_qs = Staff.objects.filter(username__iexact="staff1")
        staff_exist = staff_qs.exists() and staff_qs.count() == 1
        self.assertTrue(staff_exist)
        self.assertTrue(self.staff_a.check_password(self.staff_a_pass))

class AdminTestCase(unittest.TestCase):
    

    def test_admin_exists(self):
        admin_a_pass = 'somepasswaord'
        admin_a = TenantUser.objects.create_superuser(
            username='admin', first_name='abc123', last_name='khamban', email='adminabc123@email.com',
            password=admin_a_pass
            )
        self.admin_a_pass = admin_a_pass
        admin_a.set_password(admin_a_pass)
        #admin_a.is_superuser = True
        admin_a.save()
        self.admin_a = admin_a
        admin_exits = TenantUser.objects.filter(email__iexact="adminabc123@email.com").exists()
        self.assertTrue(admin_exits)
        
    
    def test_superuser(self):

        superuser = UserTenantPermissions.objects.filter(is_superuser=True, profile_id=1)
        is_super = TenantUser.objects.filter(usertenantpermissions__is_superuser=True)
        super_exists = is_super.exists()
        #self.assertEqual(superuser, True)              
        self.assertTrue(super_exists, True)
        

    def test_admin_password(self):
        admin_a_pass = 'somepasswaord'
        admin_qs = TenantUser.objects.get( email='adminabc123@email.com')
        self.assertTrue(admin_qs.check_password(admin_a_pass))
    

class CustomerTestCase(unittest.TestCase):
    

    def test_customer_exists(self):
        customer_a = Customer.objects.create(username='customer1', first_name='abc', last_name='kham', email='abpdpdc@email.com',)
        customer_a_pass = 'somepasswaord'
        customer_a.set_password(customer_a_pass)
        self.customer_a_pass = customer_a_pass
        customer_a.save()
        customer_qs = Customer.objects.filter(email='abpdpdc@email.com',)
        customer_qs.exists()
        customer_count = Customer.objects.all().count()
        self.assertTrue(customer_qs)
        self.assertNotEqual(customer_count, 0)

    def test_customer_password(self):
        customer_a = Customer.objects.create(username='customer1', first_name='abc', last_name='kham', email='abffffc@email.com', password='somepasswaord')
        
        customer_qs = Customer.objects.filter(email='abffffc@email.com')
        customer_exist = customer_qs.exists() and customer_qs.count() == 1
        self.assertTrue(customer_exist)
        


# Generated by Django 3.2.12 on 2023-02-08 06:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('settings', '0008_auto_20230208_0804'),
    ]

    operations = [
        migrations.AlterField(
            model_name='generalinfo',
            name='website',
            field=models.URLField(blank=True, default='https://example.com'),
        ),
    ]



import unittest
from multitenancy.subscriptions.models import Plan, ProductType, Subscription

import datetime


class TestPlan(unittest.TestCase):
    
    def test_add_feature(self):
        plan = Plan.objects.create(name="basic")
        plan.add_feature("Free domain")
        features = plan.features.all()
        for feature in features:
            self.assertEquals(feature.name,"Free domain")


        self.assertEquals(plan.name,"basic")
    

class TestProductType(unittest.TestCase):
    def test_create_defaults(self):
        defaults = ProductType.objects.create_defaults()
        producttypes = ProductType.objects.count()
        self.assertEquals(producttypes,3)

        
class TestSubscription(unittest.TestCase):

    def test_start_subscription(self):
        subscribe = Subscription.objects.create()
        subscribe.start_subscription("weekly")

        self.assertEquals(subscribe.status, "active")
        self.assertEquals(subscribe.cycle, "weekly")
        self.assertEquals(subscribe.reason, "Start Subscription")
        self.assertEquals(subscribe.renewal_date, datetime.date.today())
        self.assertEquals(subscribe.end_date, datetime.date.today() + datetime.timedelta(days=7))
        self.assertEquals(subscribe.subscription_duration, 7)
        

    def test_renew_subscription(self):
        subscribe = Subscription.objects.create()
        subscribe.start_subscription("weekly")
        subscribe.renew()

        self.assertEquals(subscribe.reason, "Subscription renewed")

    def test_cancel_subscription(self):
        subscribe = Subscription.objects.create()
        subscribe.start_subscription("weekly")
        subscribe.cancel_subscription()
        self.assertEquals(subscribe.end_date, datetime.date.today())
    
        self.assertEquals(subscribe.status, "cancelled")
        self.assertEquals(subscribe.reason, "Subscription cancelled")

    def test_activate_subscription(self):
        subscribe = Subscription.objects.create()
        subscribe.activate_subscription(30)

        self.assertEquals(subscribe.status, "active")
        self.assertEquals(subscribe.reason, "Subscription activated")
        self.assertEquals(subscribe.renewal_date, datetime.date.today())
        self.assertEquals(subscribe.end_date, datetime.date.today() + datetime.timedelta(days=30))

    def test_update_duration(self):
        subscribe = Subscription.objects.create()
        subscribe.update_duration(30)

        self.assertEquals(subscribe.subscription_duration, 30)

    def test_is_active(self):
        subscribe = Subscription.objects.create()
        self.assertEquals(subscribe.is_active, False)


class PaymentHandler:
    def __init__(self, payment_gateway):
        self.payment_gateway = payment_gateway

    def process_payment(self, payment, amount):
        """
        Process a payment through the payment gateway.
        :param payment: The Payment object representing the payment.
        :param amount: The amount of the payment.
        :return: The result of the payment processing.
        """
        # Code to communicate with the payment gateway API and process the payment goes here
        result = {'success': True, 'message': 'Payment processed successfully.'}
        return result

    def refund_payment(self, payment, amount):
        """
        Refund a payment through the payment gateway.
        :param payment: The Payment object representing the payment.
        :param amount: The amount of the refund.
        :return: The result of the refund processing.
        """
        # Code to communicate with the payment gateway API and process the refund goes here
        result = {'success': True, 'message': 'Refund processed successfully.'}
        return result





# Generated by Django 3.2.12 on 2023-01-08 14:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0006_auto_20230107_1742'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='producttype',
            name='features',
        ),
        migrations.AddField(
            model_name='plan',
            name='features',
            field=models.ManyToManyField(to='subscriptions.ProductFeature'),
        ),
        migrations.AlterField(
            model_name='producttype',
            name='name',
            field=models.CharField(choices=[('tenant', 'Tenant App'), ('domain', 'Custom Domain'), ('third_party', '3rd party App')], default='tenant', max_length=115),
        ),
    ]



# Generated by Django 3.2.12 on 2022-12-08 08:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('apps', '0008_auto_20221208_1033'),
    ]

    operations = [
        migrations.AddField(
            model_name='domain',
            name='has_custom',
            field=models.BooleanField(default=False),
        ),
    ]
